from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_auth0_helpers import auth0_alert_context, is_auth0_config_event
from pypanther.helpers.panther_base_helpers import deep_get

auth0_custom_role_created_tests: list[RuleTest] = [
    RuleTest(
        name="Other Event",
        expected_result=False,
        log={
            "data": {
                "client_id": "1HXWWGKk1Zj3JF8GvMrnCSirccDs4qvr",
                "client_name": "",
                "date": "2023-05-15 21:55:18.060000000",
                "description": "Updates risk assessment configs",
                "details": {
                    "request": {
                        "auth": {
                            "credentials": {
                                "jti": "949869e066205b5076e6df203fdd7b9b",
                                "scopes": [
                                    "create:actions",
                                    "create:actions_log_sessions",
                                    "create:authentication_methods",
                                    "create:client_credentials",
                                    "create:client_grants",
                                    "create:clients",
                                    "create:connections",
                                    "create:custom_domains",
                                    "create:email_provider",
                                    "create:email_templates",
                                    "create:guardian_enrollment_tickets",
                                    "create:integrations",
                                    "create:log_streams",
                                    "create:organization_connections",
                                    "create:organization_invitations",
                                    "create:organization_member_roles",
                                    "create:organization_members",
                                    "create:organizations",
                                    "create:requested_scopes",
                                    "create:resource_servers",
                                    "create:roles",
                                    "create:rules",
                                    "create:shields",
                                    "create:signing_keys",
                                    "create:tenant_invitations",
                                    "create:test_email_dispatch",
                                    "create:users",
                                    "delete:actions",
                                    "delete:anomaly_blocks",
                                    "delete:authentication_methods",
                                    "delete:branding",
                                    "delete:client_credentials",
                                    "delete:client_grants",
                                    "delete:clients",
                                    "delete:connections",
                                    "delete:custom_domains",
                                    "delete:device_credentials",
                                    "delete:email_provider",
                                    "delete:email_templates",
                                    "delete:grants",
                                    "delete:guardian_enrollments",
                                    "delete:integrations",
                                    "delete:log_streams",
                                    "delete:organization_connections",
                                    "delete:organization_invitations",
                                    "delete:organization_member_roles",
                                    "delete:organization_members",
                                    "delete:organizations",
                                    "delete:owners",
                                    "delete:requested_scopes",
                                    "delete:resource_servers",
                                    "delete:roles",
                                    "delete:rules",
                                    "delete:rules_configs",
                                    "delete:shields",
                                    "delete:tenant_invitations",
                                    "delete:tenant_members",
                                    "delete:tenants",
                                    "delete:users",
                                    "read:actions",
                                    "read:anomaly_blocks",
                                    "read:attack_protection",
                                    "read:authentication_methods",
                                    "read:branding",
                                    "read:checks",
                                    "read:client_credentials",
                                    "read:client_grants",
                                    "read:client_keys",
                                    "read:clients",
                                    "read:connections",
                                    "read:custom_domains",
                                    "read:device_credentials",
                                    "read:email_provider",
                                    "read:email_templates",
                                    "read:email_triggers",
                                    "read:entity_counts",
                                    "read:grants",
                                    "read:guardian_factors",
                                    "read:insights",
                                    "read:integrations",
                                    "read:log_streams",
                                    "read:logs",
                                    "read:mfa_policies",
                                    "read:organization_connections",
                                    "read:organization_invitations",
                                    "read:organization_member_roles",
                                    "read:organization_members",
                                    "read:organizations",
                                    "read:prompts",
                                    "read:requested_scopes",
                                    "read:resource_servers",
                                    "read:roles",
                                    "read:rules",
                                    "read:rules_configs",
                                    "read:shields",
                                    "read:signing_keys",
                                    "read:stats",
                                    "read:tenant_invitations",
                                    "read:tenant_members",
                                    "read:tenant_settings",
                                    "read:triggers",
                                    "read:users",
                                    "run:checks",
                                    "update:actions",
                                    "update:attack_protection",
                                    "update:authentication_methods",
                                    "update:branding",
                                    "update:client_credentials",
                                    "update:client_grants",
                                    "update:client_keys",
                                    "update:clients",
                                    "update:connections",
                                    "update:custom_domains",
                                    "update:email_provider",
                                    "update:email_templates",
                                    "update:email_triggers",
                                    "update:guardian_factors",
                                    "update:integrations",
                                    "update:log_streams",
                                    "update:mfa_policies",
                                    "update:organization_connections",
                                    "update:organizations",
                                    "update:prompts",
                                    "update:requested_scopes",
                                    "update:resource_servers",
                                    "update:roles",
                                    "update:rules",
                                    "update:rules_configs",
                                    "update:shields",
                                    "update:signing_keys",
                                    "update:tenant_members",
                                    "update:tenant_settings",
                                    "update:triggers",
                                    "update:users",
                                ],
                            },
                            "strategy": "jwt",
                            "user": {
                                "email": "user.name@yourcompany.io",
                                "name": "User Name",
                                "user_id": "google-oauth2|105261262156475850461",
                            },
                        },
                        "body": {"AfterAuthentication": False},
                        "channel": "https://manage.auth0.com/",
                        "ip": "12.12.12.12",
                        "method": "patch",
                        "path": "/api/v2/risk-assessment/config",
                        "query": {},
                        "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                    },
                    "response": {
                        "body": {
                            "AfterAuthentication": False,
                            "BeforeLoginPrompt": False,
                            "BeforeLoginPromptMonitoring": False,
                        },
                        "statusCode": 200,
                    },
                },
                "ip": "12.12.12.12",
                "log_id": "90020230515215719063964000000000000001223372037488829643",
                "type": "sapi",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                "user_id": "google-oauth2|105261262156475850461",
            },
            "log_id": "90020230515215719063964000000000000001223372037488829643",
            "p_any_ip_addresses": ["12.12.12.12"],
            "p_any_usernames": ["google-oauth2|105261262156475850461"],
            "p_event_time": "2023-05-15 21:55:18.06",
            "p_log_type": "Auth0.Events",
            "p_parse_time": "2023-05-15 21:58:28.629",
            "p_row_id": "42b742dba9d5f3f284d49b971814",
            "p_schema_version": 0,
            "p_source_id": "b9031579-b2c5-45c2-b15c-632b995a4e36",
            "p_source_label": "Org Tenant Label",
        },
    ),
    RuleTest(
        name="Custom Role Created",
        expected_result=True,
        log={
            "data": {
                "client_id": "1HXWWGKk1Zj3JF8GvMrnCSirccDs4qvr",
                "client_name": "",
                "date": "2023-05-16 17:28:11.165000000",
                "description": "Create a role",
                "details": {
                    "request": {
                        "auth": {
                            "credentials": {
                                "jti": "0107c849078d8d889af711840197ba7c",
                                "scopes": [
                                    "create:actions",
                                    "create:actions_log_sessions",
                                    "create:authentication_methods",
                                    "create:client_credentials",
                                    "create:client_grants",
                                    "create:clients",
                                    "create:connections",
                                    "create:custom_domains",
                                    "create:email_provider",
                                    "create:email_templates",
                                    "create:guardian_enrollment_tickets",
                                    "create:integrations",
                                    "create:log_streams",
                                    "create:organization_connections",
                                    "create:organization_invitations",
                                    "create:organization_member_roles",
                                    "create:organization_members",
                                    "create:organizations",
                                    "create:requested_scopes",
                                    "create:resource_servers",
                                    "create:roles",
                                    "create:rules",
                                    "create:shields",
                                    "create:signing_keys",
                                    "create:tenant_invitations",
                                    "create:test_email_dispatch",
                                    "create:users",
                                    "delete:actions",
                                    "delete:anomaly_blocks",
                                    "delete:authentication_methods",
                                    "delete:branding",
                                    "delete:client_credentials",
                                    "delete:client_grants",
                                    "delete:clients",
                                    "delete:connections",
                                    "delete:custom_domains",
                                    "delete:device_credentials",
                                    "delete:email_provider",
                                    "delete:email_templates",
                                    "delete:grants",
                                    "delete:guardian_enrollments",
                                    "delete:integrations",
                                    "delete:log_streams",
                                    "delete:organization_connections",
                                    "delete:organization_invitations",
                                    "delete:organization_member_roles",
                                    "delete:organization_members",
                                    "delete:organizations",
                                    "delete:owners",
                                    "delete:requested_scopes",
                                    "delete:resource_servers",
                                    "delete:roles",
                                    "delete:rules",
                                    "delete:rules_configs",
                                    "delete:shields",
                                    "delete:tenant_invitations",
                                    "delete:tenant_members",
                                    "delete:tenants",
                                    "delete:users",
                                    "read:actions",
                                    "read:anomaly_blocks",
                                    "read:attack_protection",
                                    "read:authentication_methods",
                                    "read:branding",
                                    "read:checks",
                                    "read:client_credentials",
                                    "read:client_grants",
                                    "read:client_keys",
                                    "read:clients",
                                    "read:connections",
                                    "read:custom_domains",
                                    "read:device_credentials",
                                    "read:email_provider",
                                    "read:email_templates",
                                    "read:email_triggers",
                                    "read:entity_counts",
                                    "read:grants",
                                    "read:guardian_factors",
                                    "read:insights",
                                    "read:integrations",
                                    "read:log_streams",
                                    "read:logs",
                                    "read:mfa_policies",
                                    "read:organization_connections",
                                    "read:organization_invitations",
                                    "read:organization_member_roles",
                                    "read:organization_members",
                                    "read:organizations",
                                    "read:prompts",
                                    "read:requested_scopes",
                                    "read:resource_servers",
                                    "read:roles",
                                    "read:rules",
                                    "read:rules_configs",
                                    "read:shields",
                                    "read:signing_keys",
                                    "read:stats",
                                    "read:tenant_invitations",
                                    "read:tenant_members",
                                    "read:tenant_settings",
                                    "read:triggers",
                                    "read:users",
                                    "run:checks",
                                    "update:actions",
                                    "update:attack_protection",
                                    "update:authentication_methods",
                                    "update:branding",
                                    "update:client_credentials",
                                    "update:client_grants",
                                    "update:client_keys",
                                    "update:clients",
                                    "update:connections",
                                    "update:custom_domains",
                                    "update:email_provider",
                                    "update:email_templates",
                                    "update:email_triggers",
                                    "update:guardian_factors",
                                    "update:integrations",
                                    "update:log_streams",
                                    "update:mfa_policies",
                                    "update:organization_connections",
                                    "update:organizations",
                                    "update:prompts",
                                    "update:requested_scopes",
                                    "update:resource_servers",
                                    "update:roles",
                                    "update:rules",
                                    "update:rules_configs",
                                    "update:shields",
                                    "update:signing_keys",
                                    "update:tenant_members",
                                    "update:tenant_settings",
                                    "update:triggers",
                                    "update:users",
                                ],
                            },
                            "strategy": "jwt",
                            "user": {
                                "email": "user.name@yourcompany.io",
                                "name": "User Name",
                                "user_id": "google-oauth2|105261262156475850461",
                            },
                        },
                        "body": {"name": "test", "description": "test"},
                        "channel": "https://manage.auth0.com/",
                        "ip": "12.12.12.12",
                        "method": "put",
                        "path": "/api/v2/guardian/policies",
                        "query": {},
                        "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                    },
                    "response": {"body": [], "statusCode": 200},
                },
                "ip": "12.12.12.12",
                "log_id": "90020230516172812328948000000000000001223372037498448373",
                "type": "sapi",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                "user_id": "google-oauth2|105261262156475850461",
            },
            "log_id": "90020230516172812328948000000000000001223372037498448373",
            "p_any_ip_addresses": ["12.12.12.12"],
            "p_any_usernames": ["google-oauth2|105261262156475850461"],
            "p_event_time": "2023-05-16 17:28:11.165",
            "p_log_type": "Auth0.Events",
            "p_parse_time": "2023-05-16 17:29:28.904",
            "p_row_id": "a2d1b1713461e7a5f1b08f991886e402",
            "p_schema_version": 0,
            "p_source_id": "b9031579-b2c5-45c2-b15c-632b995a4e36",
            "p_source_label": "Org Tenant Label",
        },
    ),
    RuleTest(
        name="Admin Role Created",
        expected_result=True,
        log={
            "data": {
                "client_id": "1HXWWGKk1Zj3JF8GvMrnCSirccDs4qvr",
                "client_name": "",
                "date": "2023-05-16 17:28:11.165000000",
                "description": "Create a role",
                "details": {
                    "request": {
                        "auth": {
                            "credentials": {
                                "jti": "0107c849078d8d889af711840197ba7c",
                                "scopes": [
                                    "create:actions",
                                    "create:actions_log_sessions",
                                    "create:authentication_methods",
                                    "create:client_credentials",
                                    "create:client_grants",
                                    "create:clients",
                                    "create:connections",
                                    "create:custom_domains",
                                    "create:email_provider",
                                    "create:email_templates",
                                    "create:guardian_enrollment_tickets",
                                    "create:integrations",
                                    "create:log_streams",
                                    "create:organization_connections",
                                    "create:organization_invitations",
                                    "create:organization_member_roles",
                                    "create:organization_members",
                                    "create:organizations",
                                    "create:requested_scopes",
                                    "create:resource_servers",
                                    "create:roles",
                                    "create:rules",
                                    "create:shields",
                                    "create:signing_keys",
                                    "create:tenant_invitations",
                                    "create:test_email_dispatch",
                                    "create:users",
                                    "delete:actions",
                                    "delete:anomaly_blocks",
                                    "delete:authentication_methods",
                                    "delete:branding",
                                    "delete:client_credentials",
                                    "delete:client_grants",
                                    "delete:clients",
                                    "delete:connections",
                                    "delete:custom_domains",
                                    "delete:device_credentials",
                                    "delete:email_provider",
                                    "delete:email_templates",
                                    "delete:grants",
                                    "delete:guardian_enrollments",
                                    "delete:integrations",
                                    "delete:log_streams",
                                    "delete:organization_connections",
                                    "delete:organization_invitations",
                                    "delete:organization_member_roles",
                                    "delete:organization_members",
                                    "delete:organizations",
                                    "delete:owners",
                                    "delete:requested_scopes",
                                    "delete:resource_servers",
                                    "delete:roles",
                                    "delete:rules",
                                    "delete:rules_configs",
                                    "delete:shields",
                                    "delete:tenant_invitations",
                                    "delete:tenant_members",
                                    "delete:tenants",
                                    "delete:users",
                                    "read:actions",
                                    "read:anomaly_blocks",
                                    "read:attack_protection",
                                    "read:authentication_methods",
                                    "read:branding",
                                    "read:checks",
                                    "read:client_credentials",
                                    "read:client_grants",
                                    "read:client_keys",
                                    "read:clients",
                                    "read:connections",
                                    "read:custom_domains",
                                    "read:device_credentials",
                                    "read:email_provider",
                                    "read:email_templates",
                                    "read:email_triggers",
                                    "read:entity_counts",
                                    "read:grants",
                                    "read:guardian_factors",
                                    "read:insights",
                                    "read:integrations",
                                    "read:log_streams",
                                    "read:logs",
                                    "read:mfa_policies",
                                    "read:organization_connections",
                                    "read:organization_invitations",
                                    "read:organization_member_roles",
                                    "read:organization_members",
                                    "read:organizations",
                                    "read:prompts",
                                    "read:requested_scopes",
                                    "read:resource_servers",
                                    "read:roles",
                                    "read:rules",
                                    "read:rules_configs",
                                    "read:shields",
                                    "read:signing_keys",
                                    "read:stats",
                                    "read:tenant_invitations",
                                    "read:tenant_members",
                                    "read:tenant_settings",
                                    "read:triggers",
                                    "read:users",
                                    "run:checks",
                                    "update:actions",
                                    "update:attack_protection",
                                    "update:authentication_methods",
                                    "update:branding",
                                    "update:client_credentials",
                                    "update:client_grants",
                                    "update:client_keys",
                                    "update:clients",
                                    "update:connections",
                                    "update:custom_domains",
                                    "update:email_provider",
                                    "update:email_templates",
                                    "update:email_triggers",
                                    "update:guardian_factors",
                                    "update:integrations",
                                    "update:log_streams",
                                    "update:mfa_policies",
                                    "update:organization_connections",
                                    "update:organizations",
                                    "update:prompts",
                                    "update:requested_scopes",
                                    "update:resource_servers",
                                    "update:roles",
                                    "update:rules",
                                    "update:rules_configs",
                                    "update:shields",
                                    "update:signing_keys",
                                    "update:tenant_members",
                                    "update:tenant_settings",
                                    "update:triggers",
                                    "update:users",
                                ],
                            },
                            "strategy": "jwt",
                            "user": {
                                "email": "user.name@yourcompany.io",
                                "name": "User Name",
                                "user_id": "google-oauth2|105261262156475850461",
                            },
                        },
                        "body": {"name": "admin", "description": "admin"},
                        "channel": "https://manage.auth0.com/",
                        "ip": "12.12.12.12",
                        "method": "put",
                        "path": "/api/v2/guardian/policies",
                        "query": {},
                        "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                    },
                    "response": {"body": [], "statusCode": 200},
                },
                "ip": "12.12.12.12",
                "log_id": "90020230516172812328948000000000000001223372037498448373",
                "type": "sapi",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                "user_id": "google-oauth2|105261262156475850461",
            },
            "log_id": "90020230516172812328948000000000000001223372037498448373",
            "p_any_ip_addresses": ["12.12.12.12"],
            "p_any_usernames": ["google-oauth2|105261262156475850461"],
            "p_event_time": "2023-05-16 17:28:11.165",
            "p_log_type": "Auth0.Events",
            "p_parse_time": "2023-05-16 17:29:28.904",
            "p_row_id": "a2d1b1713461e7a5f1b08f991886e402",
            "p_schema_version": 0,
            "p_source_id": "b9031579-b2c5-45c2-b15c-632b995a4e36",
            "p_source_label": "Org Tenant Label",
        },
    ),
    RuleTest(
        name="Admin Role Created (by name)",
        expected_result=True,
        log={
            "data": {
                "client_id": "1HXWWGKk1Zj3JF8GvMrnCSirccDs4qvr",
                "client_name": "",
                "date": "2023-05-16 17:28:11.165000000",
                "description": "Create a role",
                "details": {
                    "request": {
                        "auth": {
                            "credentials": {
                                "jti": "0107c849078d8d889af711840197ba7c",
                                "scopes": [
                                    "create:actions",
                                    "create:actions_log_sessions",
                                    "create:authentication_methods",
                                    "create:client_credentials",
                                    "create:client_grants",
                                    "create:clients",
                                    "create:connections",
                                    "create:custom_domains",
                                    "create:email_provider",
                                    "create:email_templates",
                                    "create:guardian_enrollment_tickets",
                                    "create:integrations",
                                    "create:log_streams",
                                    "create:organization_connections",
                                    "create:organization_invitations",
                                    "create:organization_member_roles",
                                    "create:organization_members",
                                    "create:organizations",
                                    "create:requested_scopes",
                                    "create:resource_servers",
                                    "create:roles",
                                    "create:rules",
                                    "create:shields",
                                    "create:signing_keys",
                                    "create:tenant_invitations",
                                    "create:test_email_dispatch",
                                    "create:users",
                                    "delete:actions",
                                    "delete:anomaly_blocks",
                                    "delete:authentication_methods",
                                    "delete:branding",
                                    "delete:client_credentials",
                                    "delete:client_grants",
                                    "delete:clients",
                                    "delete:connections",
                                    "delete:custom_domains",
                                    "delete:device_credentials",
                                    "delete:email_provider",
                                    "delete:email_templates",
                                    "delete:grants",
                                    "delete:guardian_enrollments",
                                    "delete:integrations",
                                    "delete:log_streams",
                                    "delete:organization_connections",
                                    "delete:organization_invitations",
                                    "delete:organization_member_roles",
                                    "delete:organization_members",
                                    "delete:organizations",
                                    "delete:owners",
                                    "delete:requested_scopes",
                                    "delete:resource_servers",
                                    "delete:roles",
                                    "delete:rules",
                                    "delete:rules_configs",
                                    "delete:shields",
                                    "delete:tenant_invitations",
                                    "delete:tenant_members",
                                    "delete:tenants",
                                    "delete:users",
                                    "read:actions",
                                    "read:anomaly_blocks",
                                    "read:attack_protection",
                                    "read:authentication_methods",
                                    "read:branding",
                                    "read:checks",
                                    "read:client_credentials",
                                    "read:client_grants",
                                    "read:client_keys",
                                    "read:clients",
                                    "read:connections",
                                    "read:custom_domains",
                                    "read:device_credentials",
                                    "read:email_provider",
                                    "read:email_templates",
                                    "read:email_triggers",
                                    "read:entity_counts",
                                    "read:grants",
                                    "read:guardian_factors",
                                    "read:insights",
                                    "read:integrations",
                                    "read:log_streams",
                                    "read:logs",
                                    "read:mfa_policies",
                                    "read:organization_connections",
                                    "read:organization_invitations",
                                    "read:organization_member_roles",
                                    "read:organization_members",
                                    "read:organizations",
                                    "read:prompts",
                                    "read:requested_scopes",
                                    "read:resource_servers",
                                    "read:roles",
                                    "read:rules",
                                    "read:rules_configs",
                                    "read:shields",
                                    "read:signing_keys",
                                    "read:stats",
                                    "read:tenant_invitations",
                                    "read:tenant_members",
                                    "read:tenant_settings",
                                    "read:triggers",
                                    "read:users",
                                    "run:checks",
                                    "update:actions",
                                    "update:attack_protection",
                                    "update:authentication_methods",
                                    "update:branding",
                                    "update:client_credentials",
                                    "update:client_grants",
                                    "update:client_keys",
                                    "update:clients",
                                    "update:connections",
                                    "update:custom_domains",
                                    "update:email_provider",
                                    "update:email_templates",
                                    "update:email_triggers",
                                    "update:guardian_factors",
                                    "update:integrations",
                                    "update:log_streams",
                                    "update:mfa_policies",
                                    "update:organization_connections",
                                    "update:organizations",
                                    "update:prompts",
                                    "update:requested_scopes",
                                    "update:resource_servers",
                                    "update:roles",
                                    "update:rules",
                                    "update:rules_configs",
                                    "update:shields",
                                    "update:signing_keys",
                                    "update:tenant_members",
                                    "update:tenant_settings",
                                    "update:triggers",
                                    "update:users",
                                ],
                            },
                            "strategy": "jwt",
                            "user": {
                                "email": "user.name@yourcompany.io",
                                "name": "User Name",
                                "user_id": "google-oauth2|105261262156475850461",
                            },
                        },
                        "body": {"name": "admin", "description": "some new role"},
                        "channel": "https://manage.auth0.com/",
                        "ip": "12.12.12.12",
                        "method": "put",
                        "path": "/api/v2/guardian/policies",
                        "query": {},
                        "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                    },
                    "response": {"body": [], "statusCode": 200},
                },
                "ip": "12.12.12.12",
                "log_id": "90020230516172812328948000000000000001223372037498448373",
                "type": "sapi",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                "user_id": "google-oauth2|105261262156475850461",
            },
            "log_id": "90020230516172812328948000000000000001223372037498448373",
            "p_any_ip_addresses": ["12.12.12.12"],
            "p_any_usernames": ["google-oauth2|105261262156475850461"],
            "p_event_time": "2023-05-16 17:28:11.165",
            "p_log_type": "Auth0.Events",
            "p_parse_time": "2023-05-16 17:29:28.904",
            "p_row_id": "a2d1b1713461e7a5f1b08f991886e402",
            "p_schema_version": 0,
            "p_source_id": "b9031579-b2c5-45c2-b15c-632b995a4e36",
            "p_source_label": "Org Tenant Label",
        },
    ),
]


@panther_managed
class Auth0CustomRoleCreated(Rule):
    default_description = "An Auth0 User created a role in your organization's tenant."
    display_name = "Auth0 Custom Role Created"
    default_runbook = "Assess if this was done by the user for a valid business reason. Be vigilant if a user created a role without proper authorization."
    default_reference = "https://auth0.com/docs/manage-users/access-control/configure-core-rbac/roles/create-roles"
    default_severity = Severity.HIGH
    log_types = [LogType.AUTH0_EVENTS]
    id = "Auth0.Custom.Role.Created-prototype"
    tests = auth0_custom_role_created_tests

    def rule(self, event):
        data_description = deep_get(event, "data", "description", default="<NO_DATA_DESCRIPTION_FOUND>")
        return all([data_description == "Create a role", is_auth0_config_event(event)])

    def title(self, event):
        user = deep_get(event, "data", "details", "request", "auth", "user", "email", default="<NO_USER_FOUND>")
        request_body_name = deep_get(
            event,
            "data",
            "details",
            "request",
            "body",
            "name",
            default="<NO_REQUEST_NAME_FOUND>",
        )
        request_body_description = deep_get(
            event,
            "data",
            "details",
            "request",
            "body",
            default="<NO_REQUEST_BODY_FOUND>",
        )
        if "admin" in request_body_description or "admin" in request_body_name:
            role_type = "admin"
        else:
            role_type = "custom"
        p_source_label = deep_get(event, "p_source_label", default="<NO_P_SOURCE_LABEL_FOUND>")
        return f"Auth0 User [{user}] created a role [{request_body_name}] with [{role_type}] permissions in your tenant [{p_source_label}]."

    def severity(self, event):
        request_body_name = deep_get(
            event,
            "data",
            "details",
            "request",
            "body",
            "name",
            default="<NO_REQUEST_NAME_FOUND>",
        )
        request_body_description = deep_get(event, "data", "details", "request", "body", "description", default="")
        if "admin" in request_body_description or "admin" in request_body_name:
            return "MEDIUM"
        return "LOW"

    def alert_context(self, event):
        return auth0_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get, okta_alert_context

okta_anonymizing_vpn_login_tests: list[RuleTest] = [
    RuleTest(
        name="Other Event",
        expected_result=False,
        log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 20, "lon": -25},
                    "postalCode": "12345",
                    "state": "Ohio",
                },
                "ipAddress": "1.3.2.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "requestId": "AbCdEf12G",
                    "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                    "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                },
            },
            "displaymessage": "Authentication of user via MFA",
            "eventtype": "user.session.start",
            "legacyeventtype": "core.user.factor.attempt_fail",
            "outcome": {"reason": "INVALID_CREDENTIALS", "result": "FAILURE"},
            "published": "2022-06-22 18:18:29.015",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Springfield",
                            "country": "United States",
                            "geolocation": {"lat": 20, "lon": -25},
                            "postalCode": "12345",
                            "state": "Ohio",
                        },
                        "ip": "1.3.2.4",
                        "version": "V4",
                    },
                ],
            },
            "securitycontext": {
                "asNumber": 701,
                "asOrg": "verizon",
                "domain": "verizon.net",
                "isProxy": False,
                "isp": "verizon",
            },
            "severity": "INFO",
            "target": [
                {
                    "alternateId": "peter.griffin@company.com",
                    "displayName": "Peter Griffin",
                    "id": "0002222AAAA",
                    "type": "User",
                },
            ],
            "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
            "uuid": "AbC-123-XyZ",
            "version": "0",
        },
    ),
    RuleTest(
        name="Anonymizing Proxy Used",
        expected_result=True,
        log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 20, "lon": -25},
                    "postalCode": "12345",
                    "state": "Ohio",
                },
                "ipAddress": "1.3.2.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "requestId": "AbCdEf12G",
                    "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                    "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                },
            },
            "displaymessage": "Authentication of user via MFA",
            "eventtype": "user.session.start",
            "legacyeventtype": "core.user.factor.attempt_fail",
            "outcome": {"reason": "FastPass declined phishing attempt", "result": "FAILURE"},
            "published": "2022-06-22 18:18:29.015",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Springfield",
                            "country": "United States",
                            "geolocation": {"lat": 20, "lon": -25},
                            "postalCode": "12345",
                            "state": "Ohio",
                        },
                        "ip": "1.3.2.4",
                        "version": "V4",
                    },
                ],
            },
            "securitycontext": {
                "asNumber": 701,
                "asOrg": "verizon",
                "domain": "anonymous.org",
                "isProxy": True,
                "isp": "verizon",
            },
            "severity": "INFO",
            "target": [
                {
                    "alternateId": "peter.griffin@company.com",
                    "displayName": "Peter Griffin",
                    "id": "0002222AAAA",
                    "type": "User",
                },
            ],
            "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
            "uuid": "AbC-123-XyZ",
            "version": "0",
        },
    ),
    RuleTest(
        name="Apple Private Relay Used",
        expected_result=True,
        log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 20, "lon": -25},
                    "postalCode": "12345",
                    "state": "Ohio",
                },
                "ipAddress": "1.3.2.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "requestId": "AbCdEf12G",
                    "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                    "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                },
            },
            "displaymessage": "Authentication of user via MFA",
            "eventtype": "user.session.start",
            "legacyeventtype": "core.user.factor.attempt_fail",
            "outcome": {"reason": "FastPass declined phishing attempt", "result": "FAILURE"},
            "p_enrichment": {
                "ipinfo_privacy": {
                    "client.ipAddress": {
                        "hosting": True,
                        "p_match": "1.2.3.4",
                        "proxy": False,
                        "relay": True,
                        "service": "Apple Private Relay",
                        "tor": False,
                        "vpn": False,
                    },
                },
            },
            "published": "2022-06-22 18:18:29.015",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Springfield",
                            "country": "United States",
                            "geolocation": {"lat": 20, "lon": -25},
                            "postalCode": "12345",
                            "state": "Ohio",
                        },
                        "ip": "1.3.2.4",
                        "version": "V4",
                    },
                ],
            },
            "securitycontext": {
                "asNumber": 701,
                "asOrg": "verizon",
                "domain": "anonymous.org",
                "isProxy": True,
                "isp": "verizon",
            },
            "severity": "INFO",
            "target": [
                {
                    "alternateId": "peter.griffin@company.com",
                    "displayName": "Peter Griffin",
                    "id": "0002222AAAA",
                    "type": "User",
                },
            ],
            "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
            "uuid": "AbC-123-XyZ",
            "version": "0",
        },
    ),
]


@panther_managed
class OktaAnonymizingVPNLogin(Rule):
    id = "Okta.Anonymizing.VPN.Login-prototype"
    display_name = "Okta Sign-In from VPN Anonymizer"
    log_types = [LogType.OKTA_SYSTEM_LOG]
    reports = {"MITRE ATT&CK": ["TA0006:T1556"]}
    default_severity = Severity.MEDIUM
    default_description = "A user is attempting to sign-in to Okta from a known VPN anonymizer.  The threat actor would access the compromised account using anonymizing proxy services.\n"
    default_runbook = "Restrict this access to trusted Network Zones and deny access from anonymizing proxies in policy using a Dynamic Network Zone.\n"
    default_reference = "https://sec.okta.com/articles/2023/08/cross-tenant-impersonation-prevention-and-detection\n"
    dedup_period_minutes = 360
    tests = okta_anonymizing_vpn_login_tests

    def rule(self, event):
        return event.get("eventType") == "user.session.start" and deep_get(
            event,
            "securityContext",
            "isProxy",
            default=False,
        )

    def title(self, event):
        return f"{deep_get(event, 'actor', 'displayName', default='<displayName-not-found>')} <{deep_get(event, 'actor', 'alternateId', default='alternateId-not-found')}> attempted to sign-in from anonymizing VPN with domain [{deep_get(event, 'securityContext', 'domain', default='<domain-not-found>')}]"

    def alert_context(self, event):
        return okta_alert_context(event)

    def severity(self, event):
        # If the user is using Apple Private Relay, demote the severity to INFO
        if event.deep_get("p_enrichment", "ipinfo_privacy", "client.ipAddress", "service") == "Apple Private Relay":
            return "INFO"
        # Return Medium by default
        return "MEDIUM"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_auth0_helpers import auth0_alert_context, is_auth0_config_event
from pypanther.helpers.panther_base_helpers import deep_get

auth0_user_joined_tenant_tests: list[RuleTest] = [
    RuleTest(
        name="User accepted an invitation",
        expected_result=True,
        log={
            "data": {
                "client_id": "1HXWWGKk1Zj3JF8GvMrnCSirccDs4qvr",
                "client_name": "",
                "date": "2023-05-15 16:17:00.128000000",
                "description": "Update an invitation",
                "details": {
                    "request": {
                        "auth": {
                            "credentials": {
                                "jti": "9da39f3d0b369356d2f4338ff13e4e8b",
                                "scopes": ["update:tenant_invitations"],
                            },
                            "strategy": "jwt",
                            "user": {
                                "email": "homer.simpson@yourcompany.com",
                                "name": "Homer Simpson",
                                "user_id": "google-oauth2|105261262156475850461",
                            },
                        },
                        "body": {
                            "state": "accepted",
                            "user": {
                                "email": "homer.simpson@yourcompany.com",
                                "id": "google-oauth2|105261262156475850461",
                            },
                        },
                        "channel": "https://manage.auth0.com/",
                        "ip": "12.12.12.12",
                        "method": "patch",
                        "path": "/api/v2/tenants/invitations/inv_TEyzbreI336AHrfU",
                        "query": {},
                    },
                    "response": {"body": {}, "statusCode": 200},
                },
                "ip": "12.12.12.12",
                "log_id": "90020230515161703699125000000000000001223372037485126920",
                "type": "sapi",
                "user_id": "google-oauth2|105261262156475850461",
            },
            "log_id": "90020230515161703699125000000000000001223372037485126920",
            "p_any_ip_addresses": ["12.12.12.12"],
            "p_any_usernames": ["google-oauth2|105261262156475850461"],
            "p_event_time": "2023-05-15 16:17:00.128",
            "p_log_type": "Auth0.Events",
            "p_parse_time": "2023-05-15 16:18:28.605",
            "p_row_id": "6e94415d533cdcaac7ffc79618fb9b01",
            "p_schema_version": 0,
            "p_source_id": "b9031579-b2c5-45c2-b15c-632b995a4e36",
            "p_source_label": "Org Auth0 Tenant Label",
        },
    ),
    RuleTest(
        name="User declined an invitation",
        expected_result=False,
        log={
            "data": {
                "client_id": "1HXWWGKk1Zj3JF8GvMrnCSirccDs4qvr",
                "client_name": "",
                "date": "2023-05-15 16:17:00.128000000",
                "description": "Update an invitation",
                "details": {
                    "request": {
                        "auth": {
                            "credentials": {
                                "jti": "9da39f3d0b369356d2f4338ff13e4e8b",
                                "scopes": ["update:tenant_invitations"],
                            },
                            "strategy": "jwt",
                            "user": {
                                "email": "homer.simpson@yourcompany.com",
                                "name": "Homer Simpson",
                                "user_id": "google-oauth2|105261262156475850461",
                            },
                        },
                        "body": {
                            "state": "declined",
                            "user": {
                                "email": "homer.simpson@yourcompany.com",
                                "id": "google-oauth2|105261262156475850461",
                            },
                        },
                        "channel": "https://manage.auth0.com/",
                        "ip": "12.12.12.12",
                        "method": "patch",
                        "path": "/api/v2/tenants/invitations/inv_TEyzbreI336AHrfU",
                        "query": {},
                    },
                    "response": {"body": {}, "statusCode": 200},
                },
                "ip": "12.12.12.12",
                "log_id": "90020230515161703699125000000000000001223372037485126920",
                "type": "sapi",
                "user_id": "google-oauth2|105261262156475850461",
            },
            "log_id": "90020230515161703699125000000000000001223372037485126920",
            "p_any_ip_addresses": ["12.12.12.12"],
            "p_any_usernames": ["google-oauth2|105261262156475850461"],
            "p_event_time": "2023-05-15 16:17:00.128",
            "p_log_type": "Auth0.Events",
            "p_parse_time": "2023-05-15 16:18:28.605",
            "p_row_id": "6e94415d533cdcaac7ffc79618fb9b01",
            "p_schema_version": 0,
            "p_source_id": "b9031579-b2c5-45c2-b15c-632b995a4e36",
            "p_source_label": "Org Auth0 Tenant Label",
        },
    ),
    RuleTest(
        name="Other event",
        expected_result=False,
        log={
            "data": {
                "client_id": "1HXWWGKk1Zj3JF8GvMrnCSirccDs4qvr",
                "client_name": "",
                "date": "2023-05-15 16:13:53.609000000",
                "description": "Create tenant invitations for a given client",
                "details": {
                    "request": {
                        "auth": {
                            "credentials": {
                                "jti": "dc1843dbe925a1ed2e707452c2123913",
                                "scopes": [
                                    "create:actions",
                                    "create:actions_log_sessions",
                                    "create:authentication_methods",
                                    "create:client_credentials",
                                    "create:client_grants",
                                    "create:clients",
                                    "create:connections",
                                    "create:custom_domains",
                                    "create:email_provider",
                                    "create:email_templates",
                                    "create:guardian_enrollment_tickets",
                                    "create:integrations",
                                    "create:log_streams",
                                    "create:organization_connections",
                                    "create:organization_invitations",
                                    "create:organization_member_roles",
                                    "create:organization_members",
                                    "create:organizations",
                                    "create:requested_scopes",
                                    "create:resource_servers",
                                    "create:roles",
                                    "create:rules",
                                    "create:shields",
                                    "create:signing_keys",
                                    "create:tenant_invitations",
                                    "create:test_email_dispatch",
                                    "create:users",
                                    "delete:actions",
                                    "delete:anomaly_blocks",
                                    "delete:authentication_methods",
                                    "delete:branding",
                                    "delete:client_credentials",
                                    "delete:client_grants",
                                    "delete:clients",
                                    "delete:connections",
                                    "delete:custom_domains",
                                    "delete:device_credentials",
                                    "delete:email_provider",
                                    "delete:email_templates",
                                    "delete:grants",
                                    "delete:guardian_enrollments",
                                    "delete:integrations",
                                    "delete:log_streams",
                                    "delete:organization_connections",
                                    "delete:organization_invitations",
                                    "delete:organization_member_roles",
                                    "delete:organization_members",
                                    "delete:organizations",
                                    "delete:owners",
                                    "delete:requested_scopes",
                                    "delete:resource_servers",
                                    "delete:roles",
                                    "delete:rules",
                                    "delete:rules_configs",
                                    "delete:shields",
                                    "delete:tenant_invitations",
                                    "delete:tenant_members",
                                    "delete:tenants",
                                    "delete:users",
                                    "read:actions",
                                    "read:anomaly_blocks",
                                    "read:attack_protection",
                                    "read:authentication_methods",
                                    "read:branding",
                                    "read:checks",
                                    "read:client_credentials",
                                    "read:client_grants",
                                    "read:client_keys",
                                    "read:clients",
                                    "read:connections",
                                    "read:custom_domains",
                                    "read:device_credentials",
                                    "read:email_provider",
                                    "read:email_templates",
                                    "read:email_triggers",
                                    "read:entity_counts",
                                    "read:grants",
                                    "read:guardian_factors",
                                    "read:insights",
                                    "read:integrations",
                                    "read:log_streams",
                                    "read:logs",
                                    "read:mfa_policies",
                                    "read:organization_connections",
                                    "read:organization_invitations",
                                    "read:organization_member_roles",
                                    "read:organization_members",
                                    "read:organizations",
                                    "read:prompts",
                                    "read:requested_scopes",
                                    "read:resource_servers",
                                    "read:roles",
                                    "read:rules",
                                    "read:rules_configs",
                                    "read:shields",
                                    "read:signing_keys",
                                    "read:stats",
                                    "read:tenant_invitations",
                                    "read:tenant_members",
                                    "read:tenant_settings",
                                    "read:triggers",
                                    "read:users",
                                    "run:checks",
                                    "update:actions",
                                    "update:attack_protection",
                                    "update:authentication_methods",
                                    "update:branding",
                                    "update:client_credentials",
                                    "update:client_grants",
                                    "update:client_keys",
                                    "update:clients",
                                    "update:connections",
                                    "update:custom_domains",
                                    "update:email_provider",
                                    "update:email_templates",
                                    "update:email_triggers",
                                    "update:guardian_factors",
                                    "update:integrations",
                                    "update:log_streams",
                                    "update:mfa_policies",
                                    "update:organization_connections",
                                    "update:organizations",
                                    "update:prompts",
                                    "update:requested_scopes",
                                    "update:resource_servers",
                                    "update:roles",
                                    "update:rules",
                                    "update:rules_configs",
                                    "update:shields",
                                    "update:signing_keys",
                                    "update:tenant_members",
                                    "update:tenant_settings",
                                    "update:triggers",
                                    "update:users",
                                ],
                            },
                            "strategy": "jwt",
                            "user": {
                                "email": "homer.simpson@yourcompany.io",
                                "name": "homer.simpson@yourcompany.io",
                                "user_id": "auth0|6459776e974703f3a65dc258",
                            },
                        },
                        "body": {"owners": ["marge.simpson@yourcompany.io"], "roles": ["owner"]},
                        "channel": "https://manage.auth0.com/",
                        "ip": "12.12.12.12",
                        "method": "post",
                        "path": "/api/v2/tenants/invitations",
                        "query": {},
                        "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                    },
                    "response": {
                        "body": [
                            {
                                "email": "marge.simpson@yourcompany.io",
                                "expires_at": "2023-05-18T16:13:53.600Z",
                                "invitation_id": "inv_TEyzbreI336AHrfU",
                            },
                        ],
                        "statusCode": 201,
                    },
                },
                "ip": "12.12.12.12",
                "log_id": "90020230515161358744602000000000000001223372037485092159",
                "type": "sapi",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                "user_id": "auth0|6459776e974703f3a65dc258",
            },
            "log_id": "90020230515161358744602000000000000001223372037485092159",
            "p_any_ip_addresses": ["12.12.12.12"],
            "p_any_usernames": ["auth0|6459776e974703f3a65dc258"],
            "p_event_time": "2023-05-15 16:13:53.609",
            "p_log_type": "Auth0.Events",
            "p_parse_time": "2023-05-15 16:15:28.555",
            "p_row_id": "e20ac28001d19ac6df97b99618d4a207",
            "p_schema_version": 0,
            "p_source_id": "b9031579-b2c5-45c2-b15c-632b995a4e36",
            "p_source_label": "Org Auth0 Tenant Label",
        },
    ),
]


@panther_managed
class Auth0UserJoinedTenant(Rule):
    display_name = "Auth0 User Joined Tenant"
    default_description = "User accepted invitation from Auth0 member to join an Auth0 tenant."
    id = "Auth0.User.Joined.Tenant-prototype"
    default_reference = "https://auth0.com/docs/manage-users/organizations/configure-organizations/invite-members#send-membership-invitations:~:text=.-,Send%20membership%20invitations,-You%20can"
    default_severity = Severity.INFO
    log_types = [LogType.AUTH0_EVENTS]
    tests = auth0_user_joined_tenant_tests

    def rule(self, event):
        data_description = deep_get(event, "data", "description", default="<NO_DATA_DESCRIPTION_FOUND>")
        scopes = deep_get(
            event,
            "data",
            "details",
            "request",
            "auth",
            "credentials",
            "scopes",
            default=["<NO_CREDENTIAL_SCOPE>"],
        )
        state = deep_get(event, "data", "details", "request", "body", "state", default="<NO_STATE>")
        return all(
            [
                data_description == "Update an invitation",
                "update:tenant_invitations" in scopes,
                state == "accepted",
                is_auth0_config_event(event),
            ],
        )

    def title(self, event):
        user = deep_get(event, "data", "details", "request", "auth", "user", "email", default="<NO_USER_FOUND>")
        p_source_label = deep_get(event, "p_source_label", default="<NO_P_SOURCE_LABEL_FOUND>")
        return f"Auth0 User [{user}] has accepted an invitation to join your organization's tenant [{p_source_label}]."

    def alert_context(self, event):
        return auth0_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

one_password_unusual_client_tests: list[RuleTest] = [
    RuleTest(
        name="1Password - Expected Client",
        expected_result=False,
        log={
            "uuid": "1234",
            "session_uuid": "12345",
            "timestamp": "2021-12-15 18:02:23",
            "category": "success",
            "type": "credentials_ok",
            "country": "US",
            "target_user": {"email": "homer@springfield.gov", "name": "Homer Simpson", "uuid": "1234"},
            "client": {
                "app_name": "1Password for Mac",
                "app_version": "70902005",
                "ip_address": "1.1.1.1",
                "os_name": "MacOSX",
                "os_version": "11.6.1",
                "platform_name": "US - C02FR0H8MD6P",
                "platform_version": "MacBookPro16,1",
            },
            "p_log_type": "OnePassword.SignInAttempt",
        },
    ),
    RuleTest(
        name="1Password - Bad Client",
        expected_result=True,
        log={
            "uuid": "1234",
            "session_uuid": "12345",
            "timestamp": "2021-12-15 18:02:23",
            "category": "success",
            "type": "credentials_ok",
            "country": "US",
            "target_user": {"email": "homer@springfield.gov", "name": "Homer Simpson", "uuid": "1234"},
            "client": {
                "app_name": "Bartco 1Password Manager",
                "app_version": "70902005",
                "ip_address": "1.1.1.1",
                "os_name": "MacOSX",
                "os_version": "11.6.1",
                "platform_name": "US - C02FR0H8MD6P",
                "platform_version": "MacBookPro16,1",
            },
            "p_log_type": "OnePassword.SignInAttempt",
        },
    ),
]


@panther_managed
class OnePasswordUnusualClient(Rule):
    id = "OnePassword.Unusual.Client-prototype"
    dedup_period_minutes = 120
    display_name = "Unusual 1Password Client Detected"
    log_types = [LogType.ONEPASSWORD_SIGNIN_ATTEMPT]
    default_severity = Severity.MEDIUM
    default_description = "Detects when unusual or undesirable 1Password clients access your 1Password account"
    default_reference = "https://support.1password.com/category/accounts/"
    tags = ["1Password", "Credential Access:Credentials from Password Stores"]
    reports = {"MITRE ATT&CK": ["TA0006:T1555"]}
    summary_attributes = ["p_any_ip_addresses", "p_any_emails"]
    tests = one_password_unusual_client_tests
    "\nThis rule detects unusual or unauthorized clients connecting to your 1Password account.\nIn order to get a baseline of what clients are being used in your environment run the following\nquery in Data Explorer:\n\nselect distinct client:app_name from panther_logs.public.onepassword_signinattempt\n\nThe client_allowlist variable is a collection of standard 1Password clients.\nIf this differs from your orginization's needs this rule can be edited to suit your environment\n"

    def rule(self, event):  # Used for automated account provisioning
        client_allowlist = [
            "1Password CLI",
            "1Password for Web",
            "1Password for Mac",
            "1Password SCIM Bridge",
            "1Password for Windows",
            "1Password for iOS",
            "1Password Browser Extension",
            "1Password for Android",
        ]
        return deep_get(event, "client", "app_name") not in client_allowlist

    def title(self, event):
        return f"Unusual 1Password client - {deep_get(event, 'client', 'app_name')} detected"

    def alert_context(self, event):
        context = {}
        context["user"] = deep_get(event, "target_user", "name", default="UNKNOWN_USER")
        context["user_email"] = event.udm("actor_user")
        context["ip_address"] = event.udm("source_ip")
        context["client"] = deep_get(event, "client", "app_name", default="UNKNOWN_CLIENT")
        context["OS"] = deep_get(event, "client", "os_name", default="UNKNOWN_OS")
        context["login_result"] = event.get("category")
        context["time_seen"] = event.get("timestamp")
        return context



import argparse
import importlib
import logging
import sys
from typing import Callable, Tuple

from gql.transport.aiohttp import log as aiohttp_logger

from pypanther import testing
from pypanther.custom_logging import setup_logging
from pypanther.setup_subparsers import (
    setup_get_rule_parser,
    setup_list_rules_parser,
    setup_upload_parser,
    setup_test_parser,
)
from pypanther.vendor.panther_analysis_tool import util
from pypanther.vendor.panther_analysis_tool.command import standard_args
from pypanther.vendor.panther_analysis_tool.config import dynaconf_argparse_merge, setup_dynaconf


def run():
    setup_logging()

    parser = setup_parser()
    args = parser.parse_args()
    if args.command is None:
        parser.print_help()
        return

    if args.debug:
        logging.getLogger().setLevel(logging.DEBUG)
    else:
        aiohttp_logger.setLevel(logging.WARNING)

    config_file_settings = setup_dynaconf()
    dynaconf_argparse_merge(vars(args), config_file_settings)

    try:
        return_code, out = args.func(args)
    except util.BackendNotFoundException as err:
        logging.error('Backend not found: "%s"', err)
        return 1
    except Exception as err:  # pylint: disable=broad-except
        # Catch arbitrary exceptions without printing help message
        logging.warning('Unhandled exception: "%s"', err, exc_info=err, stack_info=True)
        logging.debug("Full error traceback:", exc_info=err)
        return 1

    if return_code > 0 and out:
        logging.error(out)
    elif return_code == 0 and out:
        logging.info(out)

    sys.exit(return_code)


def setup_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        description="Command line tool for using Panther's Detections-as-Code V2.",
        prog="pypanther",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument("--debug", action="store_true", dest="debug")
    subparsers = parser.add_subparsers(dest="command", help="Available commands")

    # Upload command
    upload_parser = subparsers.add_parser(
        "upload", help="Upload a file", formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )
    standard_args.for_public_api(upload_parser, required=False)
    setup_upload_parser(upload_parser)

    # Test command
    test_parser = subparsers.add_parser(
        "test", help="run tests", formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )
    test_parser.set_defaults(func=testing.run)
    setup_test_parser(test_parser)

    # Version command
    version_parser = subparsers.add_parser(
        "version", help="version", formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )
    version_parser.set_defaults(func=version)

    # Get command
    get_parser = subparsers.add_parser(
        "get",
        help="Get the class associated with a specific id",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    get_parser.set_defaults(func=help_printer(get_parser))
    get_subparsers = get_parser.add_subparsers()
    get_rule_parser = get_subparsers.add_parser(
        name="rule",
        help="Get the class associated with a specific rule by id",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    setup_get_rule_parser(get_rule_parser)

    # List command
    list_parser = subparsers.add_parser(
        name="list", help="List managed or register content", formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )
    list_parser.set_defaults(func=help_printer(list_parser))
    list_subparsers = list_parser.add_subparsers()
    list_rules_parser = list_subparsers.add_parser(
        name="rules",
        help="List panther managed and registered rules",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    setup_list_rules_parser(list_rules_parser)

    return parser


def help_printer(parser: argparse.ArgumentParser) -> Callable[[argparse.Namespace], Tuple[int, str]]:
    """
    A helper function for printing help messages. To be used as a commands func when you want the help message
    to be printed when it is run. Useful for when things have subcommands and running the top command is meaningless.
    """

    def wrapper(_: argparse.Namespace) -> Tuple[int, str]:
        parser.print_help()
        return 0, ""

    return wrapper


def version(args):
    print(importlib.metadata.version("pypanther"))
    return 0, ""


if __name__ == "__main__":
    run()



from datetime import datetime, timedelta
from typing import Dict, Tuple

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import (
    golang_nanotime_to_python_datetime,
    panther_nanotime_to_python_datetime,
)

teleport_long_lived_certs_tests: list[RuleTest] = [
    RuleTest(
        name="A certificate was created for the default period of 1 hour",
        expected_result=False,
        log={
            "cert_type": "user",
            "cluster_name": "teleport.example.com",
            "code": "TC000I",
            "ei": 0,
            "event": "cert.create",
            "time": "2023-09-17 21:00:00.000000",
            "identity": {
                "disallow_reissue": True,
                "expires": "2023-09-17T22:00:00.444444428Z",
                "impersonator": "bot-application",
                "kubernetes_cluster": "staging",
                "kubernetes_groups": ["application"],
                "logins": ["-teleport-nologin-88888888-4444-4444-4444-222222222222", "-teleport-internal-join"],
                "prev_identity_expires": "0001-01-01T00:00:00Z",
                "roles": ["application"],
                "route_to_cluster": "teleport.example.com",
                "teleport_cluster": "teleport.example.com",
                "traits": {},
                "user": "bot-application",
            },
            "uid": "88888888-4444-4444-4444-222222222222",
        },
    ),
    RuleTest(
        name="A certificate was created for longer than the default period of 1 hour",
        expected_result=True,
        log={
            "cert_type": "user",
            "cluster_name": "teleport.example.com",
            "code": "TC000I",
            "ei": 0,
            "event": "cert.create",
            "time": "2023-09-17 21:00:00.000000",
            "identity": {
                "disallow_reissue": True,
                "expires": "2043-09-17T22:00:00.444444428Z",
                "impersonator": "bot-application",
                "kubernetes_cluster": "staging",
                "kubernetes_groups": ["application"],
                "logins": ["-teleport-nologin-88888888-4444-4444-4444-222222222222", "-teleport-internal-join"],
                "prev_identity_expires": "0001-01-01T00:00:00Z",
                "roles": ["application"],
                "route_to_cluster": "teleport.example.com",
                "teleport_cluster": "teleport.example.com",
                "traits": {},
                "user": "bot-application",
            },
            "uid": "88888888-4444-4444-4444-222222222222",
        },
    ),
]


@panther_managed
class TeleportLongLivedCerts(Rule):
    id = "Teleport.LongLivedCerts-prototype"
    display_name = "A long-lived cert was created"
    log_types = [LogType.GRAVITATIONAL_TELEPORT_AUDIT]
    tags = ["Teleport"]
    default_severity = Severity.MEDIUM
    default_description = "An unusually long-lived Teleport certificate was created"
    reports = {"MITRE ATT&CK": ["TA0003:T1098"]}
    default_reference = "https://goteleport.com/docs/management/admin/"
    default_runbook = "Teleport certificates are usually issued for a short period of time. Alert if long-lived certificates were created.\n"
    summary_attributes = ["event", "code", "time", "identity"]
    tests = teleport_long_lived_certs_tests
    PANTHER_TIME_FORMAT = "%Y-%m-%d %H:%M:%S.%f"
    # Tune this to be some Greatest Common Denominator of session TTLs for your
    # environment
    MAXIMUM_NORMAL_VALIDITY_INTERVAL = timedelta(hours=12)
    # To allow some time in between when a request is submitted and authorized
    # vs when the certificate actually gets generated. In practice, this is much
    # less than 5 seconds.
    ISSUANCE_GRACE_PERIOD = timedelta(seconds=5)
    # You can audit your logs in Panther to try and understand your role/validity
    # patterns from a known-good period of access.
    # A query example:
    # ```sql
    #  SELECT
    #     cluster_name,
    #     identity:roles,
    #     DATEDIFF('HOUR', time, identity:expires) AS validity
    #  FROM
    #     panther_logs.public.gravitational_teleportaudit
    #  WHERE
    #     p_occurs_between('2023-09-01 00:00:00','2023-10-06 21:00:00Z')
    #     AND event = 'cert.create'
    #  GROUP BY cluster_name, identity:roles, validity
    #  ORDER BY validity DESC
    # ```
    # A dictionary of:
    #  cluster names: to a dictionary of:
    #     role names: mapping to a tuple of:
    #        ( maximum usual validity, expiration datetime for this rule )
    # "teleport.example.com": {
    #     "example_role": (timedelta(hours=720), datetime(2023, 12, 01, 01, 02, 03)),
    #     "other_example_role": (timedelta(hours=720), datetime.max),
    # },
    CLUSTER_ROLE_MAX_VALIDITIES: Dict[str, Dict[str, Tuple[timedelta, datetime]]] = {}

    def rule(self, event):
        if event.get("event") != "cert.create":
            return False
        max_validity = self.MAXIMUM_NORMAL_VALIDITY_INTERVAL + self.ISSUANCE_GRACE_PERIOD
        for role in event.deep_get("identity", "roles", default=[]):
            validity, expiration = self.CLUSTER_ROLE_MAX_VALIDITIES.get(event.get("cluster_name"), {}).get(
                role,
                (None, None),
            )
            if validity and expiration:
                # Ignore exceptions that have passed their expiry date
                if datetime.utcnow() < expiration:
                    max_validity = max(max_validity, validity)
        return self.validity_interval(event) > max_validity

    def validity_interval(self, event):
        event_time = panther_nanotime_to_python_datetime(event.get("time"))
        expires = golang_nanotime_to_python_datetime(event.deep_get("identity", "expires", default=None))
        if not event_time and expires:
            return False
        interval = expires - event_time
        return interval

    def title(self, event):
        identity = event.deep_get("identity", "user", default="<Cert with no User!?>")
        return f"A Certificate for [{identity}] on [{event.get('cluster_name', '<UNKNOWN_CLUSTER>')}] has been issued for an unusually long time: {self.validity_interval(event)!r} "



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_notion_helpers import notion_alert_context

notion_workspace_scim_token_generated_tests: list[RuleTest] = [
    RuleTest(
        name="other event",
        expected_result=False,
        log={
            "event": {
                "id": "...",
                "timestamp": "2023-06-02T20:16:41.217Z",
                "workspace_id": "123",
                "actor": {
                    "id": "..",
                    "object": "user",
                    "type": "person",
                    "person": {"email": "homer.simpson@yourcompany.io"},
                },
                "ip_address": "...",
                "platform": "mac-desktop",
                "type": "workspace.content_exported",
                "workspace.content_exported": {},
            },
        },
    ),
    RuleTest(
        name="Token Generated",
        expected_result=True,
        log={
            "event": {
                "id": "...",
                "timestamp": "2023-06-02T20:21:01.873Z",
                "workspace_id": "123",
                "actor": {
                    "id": "..",
                    "object": "user",
                    "type": "person",
                    "person": {"email": "homer.simpson@yourcompany.com"},
                },
                "ip_address": "...",
                "platform": "mac-desktop",
                "type": "workspace.scim_token_generated",
                "workspace.scim_token_generated": {},
            },
        },
    ),
]


@panther_managed
class NotionWorkspaceSCIMTokenGenerated(Rule):
    id = "Notion.Workspace.SCIM.Token.Generated-prototype"
    display_name = "Notion SCIM Token Generated"
    log_types = [LogType.NOTION_AUDIT_LOGS]
    tags = ["Notion", "Application Security", "Supply Chain Attack"]
    default_description = "A Notion User generated a SCIM token."
    default_severity = Severity.MEDIUM
    default_runbook = "Possible Initial Access. Follow up with the Notion User to determine if this was done for a valid business reason."
    default_reference = "https://www.notion.so/help/provision-users-and-groups-with-scim"
    tests = notion_workspace_scim_token_generated_tests

    def rule(self, event):
        event_type = event.deep_get("event", "type", default="<NO_EVENT_TYPE_FOUND>")
        return event_type == "workspace.scim_token_generated"

    def title(self, event):
        user = event.deep_get("event", "actor", "person", "email", default="<NO_USER_FOUND>")
        workspace_id = event.deep_get("event", "workspace_id", default="<NO_WORKSPACE_ID_FOUND>")
        return f"Notion User [{user}] generated a SCIM token for workspace id [{workspace_id}]."

    def alert_context(self, event):
        return notion_alert_context(event)



"""Global helpers for Notion detections"""

from pypanther.helpers.panther_base_helpers import deep_get


def notion_alert_context(event) -> dict:
    a_c = {}
    a_c["actor"] = deep_get(event, "event", "actor", default="<NO_ACTOR_FOUND>")
    a_c["action"] = deep_get(event, "event", "type", default="<NO_ACTION_FOUND>")
    return a_c



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

netskope_many_deletes_tests: list[RuleTest] = [
    RuleTest(
        name="True positive",
        expected_result=True,
        log={
            "_id": "1e589befa3da30132362f32a",
            "_insertion_epoch_timestamp": 1702318213,
            "audit_log_event": "Deleted rbi template",
            "count": 1,
            "is_netskope_personnel": False,
            "organization_unit": "",
            "severity_level": 2,
            "timestamp": "2023-12-11 18:10:13.000000000",
            "type": "admin_audit_logs",
            "ur_normalized": "service-account",
            "user": "service-account",
        },
    ),
    RuleTest(
        name="True negative",
        expected_result=False,
        log={
            "_id": "1e589befa3da30132362f32a",
            "_insertion_epoch_timestamp": 1702318213,
            "audit_log_event": "Rest API V2 Call",
            "count": 1,
            "is_netskope_personnel": False,
            "organization_unit": "",
            "severity_level": 2,
            "supporting_data": {
                "data_type": "incidents",
                "data_values": [200, "POST", "/api/v2/incidents/uba/getuci", "trid=ccb898fgrhvdd0v0lebg"],
            },
            "timestamp": "2023-12-11 18:10:13.000000000",
            "type": "admin_audit_logs",
            "ur_normalized": "service-account",
            "user": "service-account",
        },
    ),
]


@panther_managed
class NetskopeManyDeletes(Rule):
    id = "Netskope.ManyDeletes-prototype"
    display_name = "Netskope Many Objects Deleted"
    log_types = [LogType.NETSKOPE_AUDIT]
    tags = ["Netskope", "Configuration Required", "Data Destruction"]
    reports = {"MITRE ATT&CK": ["TA0040:T1485"]}
    default_severity = Severity.HIGH
    default_description = "A user deleted a large number of objects in a short period of time."
    threshold = 10
    default_runbook = "A user deleted a large number of objects in a short period of time.  Validate that this activity is expected and authorized."
    default_reference = "https://docs.netskope.com/en/netskope-help/admin-console/administration/audit-log/"
    tests = netskope_many_deletes_tests

    def rule(self, event):
        audit_log_event = event.get("audit_log_event")
        if audit_log_event and "Delete" in audit_log_event:
            return True
        return False

    def title(self, event):
        user = event.get("user", "<USER_NOT_FOUND>")
        return f"[{user}] deleted many objects in a short time"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get, slack_alert_context

slack_audit_logs_app_removed_tests: list[RuleTest] = [
    RuleTest(
        name="App Restricted",
        expected_result=True,
        log={
            "action": "app_restricted",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "E012MH3HS94",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-panther-1",
                    "id": "T01770N79GB",
                    "name": "test-workspace-1",
                    "type": "workspace",
                },
                "ua": "Go-http-client/2.0",
            },
            "date_create": "2021-06-08 22:16:15",
            "details": {"app_owner_id": "W012J3AEWAU", "is_internal_integration": True},
            "entity": {
                "app": {
                    "id": "A012F34BFEF",
                    "is_directory_approved": False,
                    "is_distributed": False,
                    "name": "app-name",
                    "scopes": ["admin"],
                },
                "type": "app",
            },
        },
    ),
    RuleTest(
        name="App Uninstalled",
        expected_result=True,
        log={
            "action": "app_uninstalled",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "E012MH3HS94",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-panther-1",
                    "id": "T01770N79GB",
                    "name": "test-workspace-1",
                    "type": "workspace",
                },
                "ua": "Go-http-client/2.0",
            },
            "date_create": "2021-06-08 22:16:15",
            "details": {"app_owner_id": "W012J3AEWAU", "is_internal_integration": True},
            "entity": {
                "app": {
                    "id": "A012F34BFEF",
                    "is_directory_approved": False,
                    "is_distributed": False,
                    "name": "app-name",
                    "scopes": ["admin"],
                },
                "type": "app",
            },
        },
    ),
    RuleTest(
        name="App removed from workspace",
        expected_result=True,
        log={
            "action": "org_app_workspace_removed",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "E012MH3HS94",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-panther-1",
                    "id": "T01770N79GB",
                    "name": "test-workspace-1",
                    "type": "workspace",
                },
                "ua": "Go-http-client/2.0",
            },
            "date_create": "2021-06-08 22:16:15",
            "details": {"app_owner_id": "W012J3AEWAU", "is_internal_integration": True},
            "entity": {
                "app": {
                    "id": "A012F34BFEF",
                    "is_directory_approved": False,
                    "is_distributed": False,
                    "name": "app-name",
                    "scopes": ["admin"],
                },
                "type": "app",
            },
        },
    ),
    RuleTest(
        name="User Logout",
        expected_result=False,
        log={
            "action": "user_logout",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace-1",
                    "id": "T01234N56GB",
                    "name": "test-workspace-1",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
            "date_create": "2022-07-28 15:22:32",
            "entity": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
            "id": "72cac009-9eb3-4dde-bac6-ee49a32a1789",
        },
    ),
]


@panther_managed
class SlackAuditLogsAppRemoved(Rule):
    id = "Slack.AuditLogs.AppRemoved-prototype"
    display_name = "Slack App Removed"
    log_types = [LogType.SLACK_AUDIT_LOGS]
    tags = ["Slack", "Impact", "Service Stop", "Defense Evasion", "Indicator Removal", "Clear Persistence"]
    reports = {"MITRE ATT&CK": ["TA0040:T1489", "TA0005:T1070.009"]}
    default_severity = Severity.MEDIUM
    default_description = "Detects when a Slack App has been removed"
    default_reference = "https://slack.com/intl/en-gb/help/articles/360003125231-Remove-apps-and-customised-integrations-from-your-workspace"
    summary_attributes = ["p_any_ip_addresses", "p_any_emails"]
    tests = slack_audit_logs_app_removed_tests
    APP_REMOVED_ACTIONS = ["app_restricted", "app_uninstalled", "org_app_workspace_removed"]

    def rule(self, event):
        return event.get("action") in self.APP_REMOVED_ACTIONS

    def title(self, event):
        return f"Slack App [{deep_get(event, 'entity', 'app', 'name')}] Removed by [{deep_get(event, 'actor', 'user', 'name')}]"

    def alert_context(self, event):
        return slack_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_notion_helpers import notion_alert_context

notion_many_pages_deleted_tests: list[RuleTest] = [
    RuleTest(
        name="Other Event",
        expected_result=False,
        log={
            "event": {
                "id": "...",
                "timestamp": "2023-06-02T20:16:41.217Z",
                "workspace_id": "..",
                "actor": {
                    "id": "..",
                    "object": "user",
                    "type": "person",
                    "person": {"email": "homer.simpson@yourcompany.io"},
                },
                "ip_address": "...",
                "platform": "mac-desktop",
                "type": "workspace.content_exported",
                "workspace.content_exported": {},
            },
        },
    ),
    RuleTest(
        name="Many Pages Deleted",
        expected_result=True,
        log={
            "event": {
                "actor": {
                    "id": "af06b6ff-dd5e-4024-b9ef-78fe77f55884",
                    "object": "user",
                    "person": {"email": "homer.simpson@yourcompany.io"},
                    "type": "person",
                },
                "details": {
                    "parent": {"database_id": "543af759-3010-4355-a71e-4sdfs3566a", "type": "database_id"},
                    "target": {"page_id": "93cf05d3-6805-4ddc-abba-adsfjhnlkwje785", "type": "page_id"},
                },
                "id": "768873bf-6b2c-40e8-b27c-1c199c4d6ae7",
                "ip_address": "12.12.12.12",
                "platform": "web",
                "timestamp": "2023-05-24 20:17:41.905000000",
                "type": "page.deleted",
                "workspace_id": "ea65b016-6abc-4dcf-808b-sdfg445654",
            },
        },
    ),
]


@panther_managed
class NotionManyPagesDeleted(Rule):
    id = "Notion.Many.Pages.Deleted-prototype"
    display_name = "Notion Many Pages Deleted"
    log_types = [LogType.NOTION_AUDIT_LOGS]
    tags = ["Notion", "Data Security", "Data Destruction"]
    default_severity = Severity.MEDIUM
    default_description = "A Notion User deleted multiple pages."
    threshold = 10
    default_runbook = "Possible Data Destruction. Follow up with the Notion User to determine if this was done for a valid business reason."
    default_reference = "https://www.notion.so/help/duplicate-delete-and-restore-content"
    tests = notion_many_pages_deleted_tests

    def rule(self, event):
        return event.deep_get("event", "type", default="<NO_EVENT_TYPE_FOUND>") == "page.deleted"

    def title(self, event):
        user = event.deep_get("event", "actor", "person", "email", default="<NO_USER_FOUND>")
        return f"Notion User [{user}] deleted multiple pages."

    def alert_context(self, event):
        context = notion_alert_context(event)
        page_id = event.deep_get("event", "details", "target", "page_id", default="<NO_PAGE_ID_FOUND>")
        context["page_id"] = page_id
        return context



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_config import config

microsoft365_exchange_external_forwarding_tests: list[RuleTest] = [
    RuleTest(
        name="Forwarding Enabled",
        expected_result=True,
        log={
            "clientip": "1.2.3.4",
            "creationtime": "2022-12-12 22:19:00",
            "externalaccess": False,
            "id": "111-22-33",
            "objectid": "homer.simpson",
            "operation": "Set-Mailbox",
            "organizationid": "11-aa-bb",
            "organizationname": "simpsons.onmicrosoft.com",
            "originatingserver": "QWERTY (1.2.3.4)",
            "parameters": [
                {
                    "Name": "Identity",
                    "Value": "ABC1.prod.outlook.com/Microsoft Exchange Hosted Organizations/simpsons.onmicrosoft.com/homer.simpson",
                },
                {"Name": "ForwardingSmtpAddress", "Value": "smtp:hello@familyguy.com"},
                {"Name": "DeliverToMailboxAndForward", "Value": "False"},
            ],
            "recordtype": 1,
            "resultstatus": "True",
            "userid": "homer.simpson@simpsons.onmicrosoft.com",
            "userkey": "12345",
            "usertype": 2,
            "workload": "Exchange",
        },
    ),
    RuleTest(
        name="Forwarding Rule",
        expected_result=True,
        log={
            "clientip": "1.2.3.4",
            "creationtime": "2022-12-12 22:19:00",
            "externalaccess": False,
            "id": "111-22-33",
            "objectid": "homer.simpson",
            "operation": "New-InboxRule",
            "organizationid": "11-aa-bb",
            "organizationname": "simpsons.onmicrosoft.com",
            "originatingserver": "QWERTY (1.2.3.4)",
            "parameters": [
                {"Name": "AlwaysDeleteOutlookRulesBlob", "Value": "False"},
                {"Name": "Force", "Value": "False"},
                {"Name": "ForwardTo", "Value": "hello@familyguy.com"},
                {"Name": "Name", "Value": "test forwarding"},
                {"Name": "StopProcessingRules", "Value": "True"},
            ],
            "recordtype": 1,
            "resultstatus": "True",
            "userid": "homer.simpson@simpsons.onmicrosoft.com",
            "userkey": "12345",
            "usertype": 2,
            "workload": "Exchange",
        },
    ),
    RuleTest(
        name="Forwarding Rule to Allowed Domain",
        expected_result=False,
        log={
            "clientip": "1.2.3.4",
            "creationtime": "2022-12-12 22:19:00",
            "externalaccess": False,
            "id": "111-22-33",
            "objectid": "homer.simpson",
            "operation": "New-InboxRule",
            "organizationid": "11-aa-bb",
            "organizationname": "simpsons.onmicrosoft.com",
            "originatingserver": "QWERTY (1.2.3.4)",
            "parameters": [
                {"Name": "AlwaysDeleteOutlookRulesBlob", "Value": "False"},
                {"Name": "Force", "Value": "False"},
                {"Name": "ForwardTo", "Value": "hello@example.com"},
                {"Name": "Name", "Value": "test forwarding"},
                {"Name": "StopProcessingRules", "Value": "True"},
            ],
            "recordtype": 1,
            "resultstatus": "True",
            "userid": "homer.simpson@simpsons.onmicrosoft.com",
            "userkey": "12345",
            "usertype": 2,
            "workload": "Exchange",
        },
    ),
    RuleTest(
        name="Forwarding Rule to Exception",
        expected_result=False,
        log={
            "clientip": "1.2.3.4",
            "creationtime": "2022-12-12 22:19:00",
            "externalaccess": False,
            "id": "111-22-33",
            "objectid": "homer.simpson",
            "operation": "New-InboxRule",
            "organizationid": "11-aa-bb",
            "organizationname": "simpsons.onmicrosoft.com",
            "originatingserver": "QWERTY (1.2.3.4)",
            "parameters": [
                {"Name": "AlwaysDeleteOutlookRulesBlob", "Value": "False"},
                {"Name": "Force", "Value": "False"},
                {"Name": "ForwardTo", "Value": "postmaster@example.com"},
                {"Name": "Name", "Value": "test forwarding"},
                {"Name": "StopProcessingRules", "Value": "True"},
            ],
            "recordtype": 1,
            "resultstatus": "True",
            "userid": "homer.simpson@simpsons.onmicrosoft.com",
            "userkey": "12345",
            "usertype": 2,
            "workload": "Exchange",
        },
    ),
    RuleTest(
        name="Forwarding To Allowed Domain",
        expected_result=False,
        log={
            "clientip": "1.2.3.4",
            "creationtime": "2022-12-12 22:19:00",
            "externalaccess": False,
            "id": "111-22-33",
            "objectid": "homer.simpson",
            "operation": "Set-Mailbox",
            "organizationid": "11-aa-bb",
            "organizationname": "simpsons.onmicrosoft.com",
            "originatingserver": "QWERTY (1.2.3.4)",
            "parameters": [
                {
                    "Name": "Identity",
                    "Value": "ABC1.prod.outlook.com/Microsoft Exchange Hosted Organizations/simpsons.onmicrosoft.com/homer.simpson",
                },
                {"Name": "ForwardingSmtpAddress", "Value": "smtp:hello@example.com"},
                {"Name": "DeliverToMailboxAndForward", "Value": "False"},
            ],
            "recordtype": 1,
            "resultstatus": "True",
            "userid": "homer.simpson@simpsons.onmicrosoft.com",
            "userkey": "12345",
            "usertype": 2,
            "workload": "Exchange",
        },
    ),
    RuleTest(
        name="Forwarding to Exception",
        expected_result=False,
        log={
            "clientip": "1.2.3.4",
            "creationtime": "2022-12-12 22:19:00",
            "externalaccess": False,
            "id": "111-22-33",
            "objectid": "homer.simpson",
            "operation": "Set-Mailbox",
            "organizationid": "11-aa-bb",
            "organizationname": "simpsons.onmicrosoft.com",
            "originatingserver": "QWERTY (1.2.3.4)",
            "parameters": [
                {
                    "Name": "Identity",
                    "Value": "ABC1.prod.outlook.com/Microsoft Exchange Hosted Organizations/simpsons.onmicrosoft.com/homer.simpson",
                },
                {"Name": "ForwardingSmtpAddress", "Value": "smtp:postmaster@example.com"},
                {"Name": "DeliverToMailboxAndForward", "Value": "False"},
            ],
            "recordtype": 1,
            "resultstatus": "True",
            "userid": "homer.simpson@simpsons.onmicrosoft.com",
            "userkey": "12345",
            "usertype": 2,
            "workload": "Exchange",
        },
    ),
    RuleTest(
        name="Log with ForwardingAddress",
        expected_result=True,
        log={
            "AppAccessContext": {},
            "ClientIP": "20.185.225.251:6688",
            "CreationTime": "2023-10-24 13:06:33.000000000",
            "ExternalAccess": False,
            "Id": "78ab3f60-bd49-42e5-e69d-08dbd4920c3d",
            "ObjectId": "28eb696a-03f7-47bd-a07a-b09d5f6e592e",
            "Operation": "Set-Mailbox",
            "OrganizationId": "18360841-3f87-44a6-8c9a-3ffc680611a0",
            "OrganizationName": "fellowship.lotr.com",
            "OriginatingServer": "AM6PR0402MB3448 (15.20.6933.008)",
            "Parameters": [
                {"Name": "Identity", "Value": "28eb696a-03f7-47bd-a07a-b09d5f6e592e"},
                {"Name": "ForwardingAddress", "Value": "sauron@mordor.dev"},
                {"Name": "ForwardingSmtpAddress", "Value": ""},
                {"Name": "DeliverToMailboxAndForward", "Value": "False"},
            ],
            "RecordType": 1,
            "ResultStatus": "True",
            "UserId": "saurman@lotr.com",
            "UserKey": "10032002CD6B7EFD",
            "UserType": 2,
            "Workload": "Exchange",
        },
    ),
]


@panther_managed
class Microsoft365ExchangeExternalForwarding(Rule):
    default_description = "Detects creation of forwarding rule to external domains"
    display_name = "Microsoft Exchange External Forwarding"
    reports = {"MITRE ATT&CK": ["TA0009:T1114"]}
    default_reference = "https://learn.microsoft.com/en-us/microsoft-365/security/office-365-security/outbound-spam-policies-external-email-forwarding?view=o365-worldwide"
    default_severity = Severity.HIGH
    log_types = [LogType.MICROSOFT365_AUDIT_EXCHANGE]
    id = "Microsoft365.Exchange.External.Forwarding-prototype"
    tests = microsoft365_exchange_external_forwarding_tests

    def rule(self, event):
        if event.get("operation", "") in ("Set-Mailbox", "New-InboxRule"):
            for param in event.get("parameters", []):
                if param.get("Name", "") in ("ForwardingSmtpAddress", "ForwardTo", "ForwardingAddress"):
                    to_email = param.get("Value", "")
                    if (
                        to_email.lower().replace("smtp:", "")
                        in config.MS_EXCHANGE_ALLOWED_FORWARDING_DESTINATION_EMAILS
                    ):
                        return False
                    for domain in config.MS_EXCHANGE_ALLOWED_FORWARDING_DESTINATION_DOMAINS:
                        if to_email.lower().replace("smtp:", "").endswith(domain):
                            return False
                    return True
        return False

    def title(self, event):
        to_email = "<no-recipient-found>"
        for param in event.get("parameters", []):
            if param.get("Name", "") in ("ForwardingSmtpAddress", "ForwardTo"):
                to_email = param.get("Value", "")
                break
        return f"Microsoft365: External Forwarding Created From [{event.get('userid', '')}] to [{to_email}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp_base_helpers import gcp_alert_context
from pypanther.helpers.panther_base_helpers import deep_get, deep_walk

gcp_cloud_run_set_iam_policy_tests: list[RuleTest] = [
    RuleTest(
        name="GCP Run IAM Policy Set",
        expected_result=True,
        log={
            "insertId": "l3jvzyd2s2s",
            "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {
                    "principalEmail": "some.user@company.com",
                    "principalSubject": "user:some.user@company.com",
                },
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "run.services.setIamPolicy",
                        "resource": "projects/some-project/locations/us-west1/services/cloudrun-exfil",
                        "resourceAttributes": {},
                    },
                    {"granted": True, "permission": "run.services.setIamPolicy", "resourceAttributes": {}},
                ],
                "methodName": "google.cloud.run.v1.Services.SetIamPolicy",
                "request": {
                    "@type": "type.googleapis.com/google.iam.v1.SetIamPolicyRequest",
                    "policy": {"bindings": [{"members": ["user:some.user@company.com"], "role": "roles/run.invoker"}]},
                    "resource": "projects/some-project/locations/us-west1/services/cloudrun-exfil",
                },
                "requestMetadata": {
                    "callerIP": "1.2.3.4",
                    "callerSuppliedUserAgent": "(gzip),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2024-02-02T09:44:26.173186Z"},
                },
                "resourceLocation": {"currentLocations": ["us-west1"]},
                "resourceName": "projects/some-project/locations/us-west1/services/cloudrun-exfil",
                "response": {
                    "@type": "type.googleapis.com/google.iam.v1.Policy",
                    "bindings": [{"members": ["user:some.user@company.com"], "role": "roles/run.invoker"}],
                    "etag": "BwYQYvUoBxs=",
                },
                "serviceName": "run.googleapis.com",
            },
            "receiveTimestamp": "2024-02-02 09:44:26.653891982",
            "resource": {
                "labels": {
                    "configuration_name": "",
                    "location": "us-west1",
                    "project_id": "some-project",
                    "revision_name": "",
                    "service_name": "",
                },
                "type": "cloud_run_revision",
            },
            "severity": "NOTICE",
            "timestamp": "2024-02-02 09:44:26.029835000",
        },
    ),
    RuleTest(
        name="GCP Run IAM Policy Not Set",
        expected_result=False,
        log={
            "insertId": "l3jvzyd2s2s",
            "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {
                    "principalEmail": "some.user@company.com",
                    "principalSubject": "user:some.user@company.com",
                },
                "authorizationInfo": [
                    {
                        "granted": False,
                        "permission": "run.services.setIamPolicy",
                        "resource": "projects/some-project/locations/us-west1/services/cloudrun-exfil",
                        "resourceAttributes": {},
                    },
                    {"granted": False, "permission": "run.services.setIamPolicy", "resourceAttributes": {}},
                ],
                "methodName": "google.cloud.run.v1.Services.SetIamPolicy",
                "request": {
                    "@type": "type.googleapis.com/google.iam.v1.SetIamPolicyRequest",
                    "policy": {"bindings": [{"members": ["user:some.user@company.com"], "role": "roles/run.invoker"}]},
                    "resource": "projects/some-project/locations/us-west1/services/cloudrun-exfil",
                },
                "requestMetadata": "...",
                "resourceLocation": "...",
                "resourceName": "projects/some-project/locations/us-west1/services/cloudrun-exfil",
                "serviceName": "run.googleapis.com",
            },
            "receiveTimestamp": "2024-02-02 09:44:26.653891982",
            "resource": "...",
            "severity": "NOTICE",
            "timestamp": "2024-02-02 09:44:26.029835000",
        },
    ),
    RuleTest(
        name="No method provided",
        expected_result=False,
        log={
            "insertId": "l3jvzyd2s2s",
            "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {
                    "principalEmail": "some.user@company.com",
                    "principalSubject": "user:some.user@company.com",
                },
                "authorizationInfo": [
                    {
                        "granted": False,
                        "permission": "run.services.setIamPolicy",
                        "resource": "projects/some-project/locations/us-west1/services/cloudrun-exfil",
                        "resourceAttributes": {},
                    },
                    {"granted": False, "permission": "run.services.setIamPolicy", "resourceAttributes": {}},
                ],
                "request": {
                    "@type": "type.googleapis.com/google.iam.v1.SetIamPolicyRequest",
                    "policy": {"bindings": [{"members": ["user:some.user@company.com"], "role": "roles/run.invoker"}]},
                    "resource": "projects/some-project/locations/us-west1/services/cloudrun-exfil",
                },
                "requestMetadata": "...",
                "resourceLocation": "...",
                "resourceName": "projects/some-project/locations/us-west1/services/cloudrun-exfil",
                "serviceName": "run.googleapis.com",
            },
            "receiveTimestamp": "2024-02-02 09:44:26.653891982",
            "resource": "...",
            "severity": "NOTICE",
            "timestamp": "2024-02-02 09:44:26.029835000",
        },
    ),
]


@panther_managed
class GCPCloudRunSetIAMPolicy(Rule):
    log_types = [LogType.GCP_AUDIT_LOG]
    default_description = "Detects new roles granted to users to Cloud Run Services. This could potentially allow the user to perform actions within the project and its resources, which could pose a security risk."
    display_name = "GCP Cloud Run Set IAM Policy"
    id = "GCP.Cloud.Run.Set.IAM.Policy-prototype"
    default_reference = "https://cloud.google.com/run/docs/securing/managing-access"
    default_runbook = "Confirm this was authorized and necessary behavior"
    default_severity = Severity.HIGH
    tests = gcp_cloud_run_set_iam_policy_tests

    def rule(self, event):
        if deep_get(event, "severity") == "ERROR":
            return False
        method_name = deep_get(event, "protoPayload", "methodName", default="")
        if not method_name.endswith("Services.SetIamPolicy"):
            return False
        authorization_info = deep_walk(event, "protoPayload", "authorizationInfo")
        if not authorization_info:
            return False
        for auth in authorization_info:
            if auth.get("permission") == "run.services.setIamPolicy" and auth.get("granted") is True:
                return True
        return False

    def title(self, event):
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        resource = deep_get(event, "resource", "resourceName", default="<RESOURCE_NOT_FOUND>")
        assigned_role = deep_walk(event, "protoPayload", "response", "bindings", "role")
        project_id = deep_get(event, "resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>")
        return f"[GCP]: [{actor}] was granted access to [{resource}] service with the [{assigned_role}] role in project [{project_id}]"

    def alert_context(self, event):
        context = gcp_alert_context(event)
        context["assigned_role"] = deep_walk(event, "protoPayload", "response", "bindings", "role")
        return context



import argparse
import collections
import json
import logging
import os
import traceback
from typing import Any, Tuple

from pypanther import cli_output, display
from pypanther.base import Rule, RuleTestResult
from pypanther.cache import DATA_MODEL_CACHE
from pypanther.import_main import NoMainModuleError, import_main
from pypanther.registry import registered_rules

INDENT = " " * 2


class TestResults:
    passed_rule_tests: dict[str, list[RuleTestResult]]
    """dict of rule ids to the tests that passed on the rule"""

    failed_rule_tests: dict[str, list[RuleTestResult]]
    """dict of rule ids to the tests that failed on the rule"""

    skipped_rules: list[str] = []
    """list of rule ids that had no tests"""

    def __init__(self):
        self.passed_rule_tests = collections.defaultdict(list)
        self.failed_rule_tests = collections.defaultdict(list)
        self.skipped_rules = []

    def add_test_results(self, rule_id: str, results: list[RuleTestResult]) -> None:
        if len(results) == 0:
            self.skipped_rules.append(rule_id)

        for result in results:
            if result.passed:
                self.passed_rule_tests[rule_id].append(result)
            else:
                self.failed_rule_tests[rule_id].append(result)

    def num_passed_tests(self) -> int:
        return sum([len(v) for _, v in self.passed_rule_tests.items()])

    def num_failed_tests(self) -> int:
        return sum([len(v) for _, v in self.failed_rule_tests.items()])

    def num_skipped_rules(self) -> int:
        return len(self.skipped_rules)

    def total_tests(self) -> int:
        return self.num_passed_tests() + self.num_failed_tests()

    def had_failed_tests(self) -> bool:
        return len(self.failed_rule_tests) > 0

    def total_rules(self) -> int:
        return len(
            {k for k in list(self.failed_rule_tests.keys()) + list(self.passed_rule_tests.keys()) + self.skipped_rules},
        )

    def num_passed_rules(self) -> int:
        return len({k for k in self.passed_rule_tests if k not in self.failed_rule_tests})

    def num_failed_rules(self) -> int:
        return len(self.failed_rule_tests)

    def all_rule_tests(self) -> dict[str, list[RuleTestResult]]:
        rule_test_results = collections.defaultdict(list)

        for rule_id, tests in self.passed_rule_tests.items():
            rule_test_results[rule_id].extend(tests)
        for rule_id, tests in self.failed_rule_tests.items():
            rule_test_results[rule_id].extend(tests)
        for rule_id in self.skipped_rules:
            rule_test_results[rule_id] = []

        return rule_test_results


def run(args: argparse.Namespace) -> Tuple[int, str]:
    try:
        import_main(os.getcwd(), "main")
    except NoMainModuleError:
        logging.error("No main.py found")  # noqa: TRY400
        return 1, ""

    test_results = run_tests(args)

    if args.output == display.OUTPUT_TYPE_JSON:
        print(
            json.dumps(
                test_output_dict(test_results, args.verbose),
                indent=display.JSON_INDENT_LEVEL,
            ),
        )

    if test_results.had_failed_tests():
        return 1, ""

    return 0, ""


def run_tests(args: argparse.Namespace) -> TestResults:
    test_results = TestResults()

    for rule in registered_rules(
        log_types=args.log_types if hasattr(args, "log_types") else None,
        id=args.id if hasattr(args, "id") else None,
        create_alert=args.create_alert if hasattr(args, "create_alert") else None,
        dedup_period_minutes=args.dedup_period_minutes if hasattr(args, "dedup_period_minutes") else None,
        display_name=args.display_name if hasattr(args, "display_name") else None,
        enabled=args.enabled if hasattr(args, "enabled") else None,
        summary_attributes=args.summary_attributes if hasattr(args, "summary_attributes") else None,
        threshold=args.threshold if hasattr(args, "threshold") else None,
        tags=args.tags if hasattr(args, "tags") else None,
        default_severity=args.default_severity if hasattr(args, "default_severity") else None,
        default_description=args.default_description if hasattr(args, "default_description") else None,
        default_reference=args.default_reference if hasattr(args, "default_reference") else None,
        default_runbook=args.default_runbook if hasattr(args, "default_runbook") else None,
        default_destinations=args.default_destinations if hasattr(args, "default_destinations") else None,
    ):
        results = rule.run_tests(DATA_MODEL_CACHE.data_model_of_logtype)
        test_results.add_test_results(rule.id, results)

        if args.output == display.OUTPUT_TYPE_TEXT:
            # intent here is to give the user more interactive feedback by printing
            # the tests as they are running instead of waiting until the very end.
            print_rule_test_results(args.verbose, rule.id, results)

    if args.output == display.OUTPUT_TYPE_TEXT:
        print_failed_test_summary(test_results)
        print_test_summary(test_results)

    return test_results


def test_output_dict(test_results: TestResults, verbose: bool) -> dict:
    return {
        "test_results": get_test_results_as_dict(test_results, verbose),
        "failed_tests_summary": get_failed_test_summary_as_dict(test_results),
        "test_summary": get_test_summary_as_dict(test_results),
    }


def get_test_results_as_dict(test_results: TestResults, verbose: bool) -> dict[str, Any]:
    test_results_dict = {
        rule_id: [
            {
                "test_name": result.test.name,
                "passed": result.passed,
                "exceptions": [
                    {
                        "func": func,
                        "exception": str(exc),
                        "stacktrace": "".join(traceback.format_exception(exc)),
                    }
                    for func, exc in get_rule_exceptions(result).items()
                    if exc is not None
                ],
                "failed_results": [
                    {
                        "func": func,
                        "expected": exp,
                        "output": exp,
                        "matched": match,
                    }
                    for func, exp, out, match in get_rule_results(result)
                    if not match and exp is not None
                ],
            }
            for result in results
        ]
        for rule_id, results in test_results.all_rule_tests().items()
    }

    if not verbose:
        # remove passing test results
        test_results_dict = {
            rule_id: [result for result in results if not result["passed"]]
            for rule_id, results in test_results_dict.items()
        }
        # prune empty lists because all tests passed or had no tests
        test_results_dict = {rule_id: results for rule_id, results in test_results_dict.items() if len(results) > 0}
        # remove stacktraces from exceptions
        for _, results in test_results_dict.items():
            for result in results:
                for exc_dict in result["exceptions"]:  # type: ignore
                    del exc_dict["stacktrace"]

    return test_results_dict


def get_failed_test_summary_as_dict(test_results: TestResults) -> list[dict[str, Any]]:
    return [
        {
            "rule_id": rule_id,
            "num_failed_tests": len(failed_tests),
            "failed_tests": [failed_test.test.name for failed_test in failed_tests],
        }
        for rule_id, failed_tests in test_results.failed_rule_tests.items()
    ]


def get_test_summary_as_dict(test_results: TestResults) -> dict[str, Any]:
    return {
        "skipped_rules": test_results.num_skipped_rules(),
        "passed_rules": test_results.num_passed_rules(),
        "failed_rules": test_results.num_failed_rules(),
        "total_rules": test_results.total_rules(),
        "passed_tests": test_results.num_passed_tests(),
        "failed_tests": test_results.num_failed_tests(),
        "total_tests": test_results.total_tests(),
    }


def print_rule_test_results(verbose: bool, rule_id: str, results: list[RuleTestResult]) -> None:
    if verbose or any(not result.passed for result in results):
        print(cli_output.header(rule_id) + ":")

    if len(results) == 0 and verbose:
        print(INDENT, "SKIP:", "rule had no tests")

    for result in results:
        if result.passed and verbose:
            print(INDENT, cli_output.success("PASS") + ":", result.test.name)

        elif not result.passed:
            print(INDENT, cli_output.bold(cli_output.failed("FAIL")) + ":", result.test.name)

            exceptions = get_rule_exceptions(result)
            for func, exc in exceptions.items():
                if exc is not None:
                    print(INDENT * 2, "-", f"Exception occurred in {func}(){': ' + str(exc) if str(exc) else ''}")
                    if verbose:
                        print_failed_test_exception(exc)

            for func, exp, out, match in get_rule_results(result):
                if exp is not None and exceptions.get(func, None) is None and not match:
                    print(INDENT * 2, "-", f"Expected {func}() to return '{exp}', but got '{out}'")

    if verbose or any(not result.passed for result in results):
        print()  # new line


def print_failed_test_exception(exc: Exception) -> None:
    print()  # new line
    for multi_line in traceback.format_exception(exc):
        for line in multi_line.split("\n"):
            if len(line) > 0:
                print(INDENT * 6, cli_output.failed(line))
    print()  # new line


def print_failed_test_summary(test_results: TestResults) -> None:
    if not test_results.had_failed_tests():
        return

    print(cli_output.header("Failed Tests") + ":")

    for i, failure in enumerate(test_results.failed_rule_tests.items(), start=1):
        rule_id, failed_tests = failure
        print(INDENT, str(i) + ".", cli_output.bold(rule_id) + ":")

        for failed_test in failed_tests:
            print(INDENT * 2, "-", failed_test.test.name)

    print()  # new line


def print_test_summary(test_results: TestResults) -> None:
    print(cli_output.header("Test Summary") + ":")

    print(INDENT, f"Skipped rules: {test_results.num_skipped_rules():>3}")
    print(INDENT, f"Passed rules:  {test_results.num_passed_rules():>3}")
    print(INDENT, cli_output.underline(f"Failed rules:  {test_results.num_failed_rules():>3}"))
    print(INDENT, f"Total rules:   {test_results.total_rules():>3}")
    print()  # new line

    print(INDENT, f"Passed tests:  {test_results.num_passed_tests():>3}")
    print(INDENT, cli_output.underline(f"Failed tests:  {test_results.num_failed_tests():>3}"))
    print(INDENT, f"Total tests:   {test_results.total_tests():>3}")


def get_rule_exceptions(result: RuleTestResult) -> dict[str, Exception]:
    return {
        "rule": result.detection_result.detection_exception,
        Rule.title.__name__: result.detection_result.title_exception,
        Rule.description.__name__: result.detection_result.description_exception,
        Rule.reference.__name__: result.detection_result.reference_exception,
        Rule.severity.__name__: result.detection_result.severity_exception,
        Rule.runbook.__name__: result.detection_result.runbook_exception,
        Rule.destinations.__name__: result.detection_result.destinations_exception,
        Rule.dedup.__name__: result.detection_result.dedup_exception,
        Rule.alert_context.__name__: result.detection_result.alert_context_exception,
    }


def get_rule_results(result: RuleTestResult) -> list[Tuple[str, Any, Any, bool]]:
    t = result.test
    r = result.detection_result
    return [
        ("rule", t.expected_result, r.detection_output, t.expected_result == r.detection_output),
        (Rule.title.__name__, t.expected_title, r.title_output, t.expected_title == r.title_output),
        (
            Rule.description.__name__,
            t.expected_description,
            r.description_output,
            t.expected_description == r.description_output,
        ),
        (Rule.reference.__name__, t.expected_reference, r.reference_output, t.expected_reference == r.reference_output),
        (Rule.severity.__name__, t.expected_severity, r.severity_output, t.expected_severity == r.severity_output),
        (Rule.runbook.__name__, t.expected_runbook, r.runbook_output, t.expected_runbook == r.runbook_output),
        (Rule.dedup.__name__, t.expected_dedup, r.dedup_output, t.expected_dedup == r.dedup_output),
        (
            Rule.alert_context.__name__,
            t.expected_alert_context,
            r.alert_context_output,
            t.expected_alert_context == json.loads(r.alert_context_output),
        ),
    ]


def print_test_results_json(test_results: TestResults) -> None:
    print()



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

google_workspace_apps_marketplace_new_domain_application_tests: list[RuleTest] = [
    RuleTest(
        name="Change Email Setting Default",
        expected_result=False,
        log={
            "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "12345"},
            "id": {
                "applicationName": "admin",
                "customerId": "D1234",
                "time": "2022-12-10 23:33:04.667000000",
                "uniqueQualifier": "-12345",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "CHANGE_EMAIL_SETTING",
            "parameters": {
                "NEW_VALUE": "1",
                "OLD_VALUE": "DEFAULT",
                "ORG_UNIT_NAME": "EXAMPLE IO",
                "SETTING_NAME": "ENABLE_G_SUITE_MARKETPLACE",
            },
            "type": "EMAIL_SETTINGS",
        },
    ),
    RuleTest(
        name="DocuSign for Google",
        expected_result=True,
        log={
            "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "12345"},
            "id": {
                "applicationName": "admin",
                "customerId": "D12345",
                "time": "2022-12-10 23:05:39.508000000",
                "uniqueQualifier": "-12345",
            },
            "kind": "admin#reports#activity",
            "name": "ADD_APPLICATION",
            "parameters": {
                "APP_ID": "469176070494",
                "APPLICATION_ENABLED": "true",
                "APPLICATION_NAME": "DocuSign eSignature for Google",
            },
            "type": "DOMAIN_SETTINGS",
        },
    ),
    RuleTest(
        name="Microsoft Apps for Google",
        expected_result=True,
        log={
            "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "12345"},
            "id": {
                "applicationName": "admin",
                "customerId": "D12345",
                "time": "2022-12-10 23:05:39.508000000",
                "uniqueQualifier": "-12345",
            },
            "kind": "admin#reports#activity",
            "name": "ADD_APPLICATION",
            "parameters": {
                "APP_ID": "469176070494",
                "APPLICATION_ENABLED": "true",
                "APPLICATION_NAME": "Microsoft Applications for Google",
            },
            "type": "DOMAIN_SETTINGS",
        },
    ),
    RuleTest(
        name="ListObject Type",
        expected_result=False,
        log={
            "actor": {"email": "user@example.io", "profileId": "118111111111111111111"},
            "id": {
                "applicationName": "drive",
                "customerId": "D12345",
                "time": "2022-12-20 17:27:47.080000000",
                "uniqueQualifier": "-7312729053723258069",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "rename",
            "parameters": {
                "actor_is_collaborator_account": None,
                "billable": True,
                "doc_id": "1GGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGG",
                "doc_title": "Document Title- Found Here",
                "doc_type": "presentation",
                "is_encrypted": None,
                "new_value": ["Document Title- Found Here"],
                "old_value": ["Document Title- Old"],
                "owner": "user@example.io",
                "owner_is_shared_drive": None,
                "owner_is_team_drive": None,
                "primary_event": True,
                "visibility": "private",
            },
            "type": "access",
        },
    ),
]


@panther_managed
class GoogleWorkspaceAppsMarketplaceNewDomainApplication(Rule):
    default_description = (
        "A Google Workspace User configured a new domain application from the Google Workspace Apps Marketplace."
    )
    display_name = "Google Workspace Apps Marketplace New Domain Application"
    default_runbook = "Confirm this was the intended behavior."
    default_reference = "https://developers.google.com/workspace/marketplace/overview"
    default_severity = Severity.MEDIUM
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    id = "Google.Workspace.Apps.Marketplace.New.Domain.Application-prototype"
    tests = google_workspace_apps_marketplace_new_domain_application_tests

    def rule(self, event):
        # Return True to match the log event and trigger an alert.
        return (
            event.get("name") == "ADD_APPLICATION"
            and event.get("parameters", {}).get("APPLICATION_ENABLED", "<NO_APPLICATION_FOUND>") == "true"
        )

    def title(self, event):
        # (Optional) Return a string which will be shown as the alert title.
        # If no 'dedup' function is defined, the return value of this method
        # will act as deduplication string.
        return f"Google Workspace User [{event.get('actor', {}).get('email', '<NO_EMAIL_PROVIDED>')}] enabled a new Google Workspace Marketplace application [{event.get('parameters', {}).get('APPLICATION_NAME', '<NO_APPLICATION_NAME_FOUND>')}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import slack_alert_context

slack_audit_logs_dlp_modified_tests: list[RuleTest] = [
    RuleTest(
        name="Native DLP Rule Deactivated",
        expected_result=True,
        log={
            "action": "native_dlp_rule_deactivated",
            "actor": {
                "type": "user",
                "user": {"email": "user@example.com", "id": "A012B3CDEFG", "name": "username", "team": "T01234N56GB"},
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace",
                    "id": "T01234N56GB",
                    "name": "test-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
        },
    ),
    RuleTest(
        name="Native DLP Violation Deleted",
        expected_result=True,
        log={
            "action": "native_dlp_violation_deleted",
            "actor": {
                "type": "user",
                "user": {"email": "user@example.com", "id": "A012B3CDEFG", "name": "username", "team": "T01234N56GB"},
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace",
                    "id": "T01234N56GB",
                    "name": "test-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
        },
    ),
    RuleTest(
        name="User Logout",
        expected_result=False,
        log={
            "action": "user_logout",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace-1",
                    "id": "T01234N56GB",
                    "name": "test-workspace-1",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
            "date_create": "2022-07-28 15:22:32",
            "entity": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
            "id": "72cac009-9eb3-4dde-bac6-ee49a32a1789",
        },
    ),
]


@panther_managed
class SlackAuditLogsDLPModified(Rule):
    id = "Slack.AuditLogs.DLPModified-prototype"
    display_name = "Slack DLP Modified"
    log_types = [LogType.SLACK_AUDIT_LOGS]
    tags = ["Slack", "Defense Evasion", "Impair Defenses", "Disable or Modify Tools", "Indicator Removal"]
    reports = {"MITRE ATT&CK": ["TA0005:T1562.001", "TA0005:T1070"]}
    default_severity = Severity.HIGH
    default_description = (
        "Detects when a Data Loss Prevention (DLP) rule has been deactivated or a violation has been deleted\n"
    )
    default_reference = "https://slack.com/intl/en-gb/help/articles/12914005852819-Slack-Connect--Data-loss-prevention"
    summary_attributes = ["action", "p_any_ip_addresses", "p_any_emails"]
    tests = slack_audit_logs_dlp_modified_tests
    DLP_ACTIONS = ["native_dlp_rule_deactivated", "native_dlp_violation_deleted"]
    # DLP violations can be removed by security engineers in the case of FPs
    # We still want to alert on these, however those should not constitute a High severity

    def rule(self, event):
        return event.get("action") in self.DLP_ACTIONS

    def title(self, event):
        if event.get("action") == "native_dlp_rule_deactivated":
            return "Slack DLP Rule Deactivated"
        return "Slack DLP Violation Deleted"

    def severity(self, event):
        if event.get("action") == "native_dlp_violation_deleted":
            return "Medium"
        return "High"

    def alert_context(self, event):
        return slack_alert_context(event)



"""Used to simplify the use of IPInfo Data in Rules"""

from typing import Union

from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_lookuptable_helpers import LookupTableMatches

IPINFO_LOCATION_LUT_NAME = "ipinfo_location"
IPINFO_ASN_LUT_NAME = "ipinfo_asn"
IPINFO_PRIVACY_LUT_NAME = "ipinfo_privacy"


# pylint: disable=multiple-statements
class PantherIPInfoException(Exception): ...


class IPInfoLocation(LookupTableMatches):
    """Helper to get IPInfo location information for enriched fields"""

    def __init__(self, event):
        super().__init__()
        super()._register(event, IPINFO_LOCATION_LUT_NAME)

    def city(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "city")

    def country(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "country")

    def latitude(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "lat")

    def longitude(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "lng")

    def postal_code(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "postal_code")

    def region(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "region")

    def region_code(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "region_code")

    def timezone(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "timezone")

    def context(self, match_field: str) -> object:
        return {
            "City": self.city(match_field),
            "Country": self.country(match_field),
            "Latitude": self.latitude(match_field),
            "Longitude": self.longitude(match_field),
            "PostalCode": self.postal_code(match_field),
            "Region": self.region(match_field),
            "RegionCode": self.region_code(match_field),
            "Timezone": self.timezone(match_field),
        }


class IPInfoASN(LookupTableMatches):
    """Helper to get IPInfo ASN information for enriched fields"""

    def __init__(self, event):
        super().__init__()
        super()._register(event, IPINFO_ASN_LUT_NAME)

    def asn(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "asn")

    def domain(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "domain")

    def name(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "name")

    def route(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "route")

    def type(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "type")

    def context(self, match_field: str) -> object:
        return {
            "ASN": self.asn(match_field),
            "Domain": self.domain(match_field),
            "Name": self.name(match_field),
            "Route": self.route(match_field),
            "Type": self.type(match_field),
        }


class IPInfoPrivacy(LookupTableMatches):
    """Helper to get IPInfo Privacy information for enriched fields"""

    def __init__(self, event):
        super().__init__()
        super()._register(event, IPINFO_PRIVACY_LUT_NAME)

    def hosting(self, match_field: str) -> Union[bool, list]:
        return self._lookup(match_field, "hosting")

    def proxy(self, match_field: str) -> Union[bool, list]:
        return self._lookup(match_field, "proxy")

    def tor(self, match_field: str) -> Union[bool, list]:
        return self._lookup(match_field, "tor")

    def vpn(self, match_field: str) -> Union[bool, list]:
        return self._lookup(match_field, "vpn")

    def relay(self, match_field: str) -> Union[bool, list]:
        return self._lookup(match_field, "relay")

    def service(self, match_field: str) -> Union[list[str], str]:
        return self._lookup(match_field, "service")

    def context(self, match_field: str) -> object:
        return {
            "Hosting": self.hosting(match_field),
            "Proxy": self.proxy(match_field),
            "Tor": self.tor(match_field),
            "VPN": self.vpn(match_field),
            "Relay": self.relay(match_field),
            "Service": self.service(match_field),
        }


def get_ipinfo_location(event):
    """Returns an IPInfoLocation object for the event or None if it is not available"""
    if deep_get(event, "p_enrichment", IPINFO_LOCATION_LUT_NAME):
        return IPInfoLocation(event)
    return None


def get_ipinfo_asn(event):
    """Returns an IPInfoASN object for the event or None if it is not available"""
    if deep_get(event, "p_enrichment", IPINFO_ASN_LUT_NAME):
        return IPInfoASN(event)
    return None


def get_ipinfo_privacy(event):
    """Returns an IPInfoPrivacy object for the event or None if it is not available"""
    if deep_get(event, "p_enrichment", IPINFO_PRIVACY_LUT_NAME):
        return IPInfoPrivacy(event)
    return None


def geoinfo_from_ip(event, match_field: str):
    """
    Returns a dictionary with geolocation information that is the same format as
    panther_oss_helper.geoinfo_from_ip() with the following differences:

    - instead of providing the ip, you must provide the event and the match_field
    - the fields "hostname" and "anycast" are not included in the return object
    """
    location = get_ipinfo_location(event)
    asn = get_ipinfo_asn(event)
    if location is None or asn is None:
        raise PantherIPInfoException("Please enable both IPInfo Location and ASN Enrichment Providers")

    if deep_get(asn.lut_matches, match_field) is None or deep_get(location.lut_matches, match_field) is None:
        raise PantherIPInfoException(f"IPInfo is not configured on the provided match_field: {match_field}")

    return {
        "ip": event.get(match_field),
        "city": location.city(match_field),
        "region": location.region(match_field),
        "country": location.country(match_field),
        "loc": f"{location.latitude(match_field)},{location.longitude(match_field)}",
        "org": f"{asn.asn(match_field)} {asn.name(match_field)}",
        "postal": location.postal_code(match_field),
        "timezone": location.timezone(match_field),
    }



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

netskope_admin_logged_out_login_failures_tests: list[RuleTest] = [
    RuleTest(
        name="True positive",
        expected_result=True,
        log={
            "_id": "e5ca619b059fccdd0cfd9398",
            "_insertion_epoch_timestamp": 1702308331,
            "audit_log_event": "Admin logged out because of successive login failures",
            "count": 1,
            "is_netskope_personnel": True,
            "organization_unit": "",
            "severity_level": 2,
            "supporting_data": {"data_type": "user", "data_values": ["11.22.33.44", "adminsupport@netskope.com"]},
            "timestamp": "2023-12-11 15:25:31.000000000",
            "type": "admin_audit_logs",
            "ur_normalized": "adminsupport@netskope.com",
            "user": "adminsupport@netskope.com",
        },
    ),
    RuleTest(
        name="True negative",
        expected_result=False,
        log={
            "_id": "1e589befa3da30132362f32a",
            "_insertion_epoch_timestamp": 1702318213,
            "audit_log_event": "Rest API V2 Call",
            "count": 1,
            "is_netskope_personnel": False,
            "organization_unit": "",
            "severity_level": 2,
            "supporting_data": {
                "data_type": "incidents",
                "data_values": [200, "POST", "/api/v2/incidents/uba/getuci", "trid=ccb898fgrhvdd0v0lebg"],
            },
            "timestamp": "2023-12-11 18:10:13.000000000",
            "type": "admin_audit_logs",
            "ur_normalized": "service-account",
            "user": "service-account",
        },
    ),
]


@panther_managed
class NetskopeAdminLoggedOutLoginFailures(Rule):
    id = "Netskope.AdminLoggedOutLoginFailures-prototype"
    display_name = "Admin logged out because of successive login failures"
    log_types = [LogType.NETSKOPE_AUDIT]
    tags = ["Netskope", "Brute Force"]
    reports = {"MITRE ATT&CK": ["TA0006:T1110"]}
    default_severity = Severity.MEDIUM
    default_description = "An admin was logged out because of successive login failures."
    default_runbook = "An admin was logged out because of successive login failures.  This could indicate brute force activity against this account."
    default_reference = "https://docs.netskope.com/en/netskope-help/admin-console/administration/audit-log/"
    tests = netskope_admin_logged_out_login_failures_tests

    def rule(self, event):
        if event.get("audit_log_event") == "Admin logged out because of successive login failures":
            return True
        return False

    def title(self, event):
        user = event.get("user", "<USER_NOT_FOUND>")
        return f"Admin [{user}] was logged out because of successive login failures"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

osquery_mac_unwanted_chrome_extensions_tests: list[RuleTest] = [
    RuleTest(
        name="Unwanted Extension Detected",
        expected_result=True,
        log={
            "action": "added",
            "calendarTime": "Tue Sep 11 16:14:21 2018 UTC",
            "columns": {
                "build_distro": "10.12",
                "build_platform": "darwin",
                "config_hash": "1111",
                "config_valid": "1",
                "counter": "14",
                "global_state": "0",
                "extensions": "active",
                "instance_id": "1111",
                "pid": "223",
                "port": "22",
                "resident_size": "54894592",
                "start_time": "1536634519",
                "system_time": "12472",
                "user_time": "31800",
                "uuid": "37821E12-CC8A-5AA3-A90C-FAB28A5BF8F9",
                "version": "Not Supported",
                "watcher": "92",
            },
            "counter": "255",
            "decorations": {"host_uuid": "1111", "environment": "corp"},
            "epoch": "0",
            "hostIdentifier": "test.lan",
            "log_type": "result",
            "name": "pack_unwanted-chrome-extensions_pup1",
            "unixTime": "1536682461",
        },
    ),
    RuleTest(
        name="No Unwanted Chrome Extension Detected",
        expected_result=False,
        log={
            "action": "added",
            "calendarTime": "Tue Sep 11 16:14:21 2018 UTC",
            "columns": {
                "build_distro": "10.12",
                "build_platform": "darwin",
                "config_hash": "1111",
                "config_valid": "1",
                "counter": "14",
                "global_state": "2",
                "extensions": "active",
                "instance_id": "1111",
                "pid": "223",
                "port": "443",
                "resident_size": "54894592",
                "start_time": "1536634519",
                "system_time": "12472",
                "user_time": "31800",
                "uuid": "37821E12-CC8A-5AA3-A90C-FAB28A5BF8F9",
                "version": "10.14.2",
                "watcher": "92",
            },
            "counter": "255",
            "decorations": {"host_uuid": "1111", "environment": "corp"},
            "epoch": "0",
            "hostIdentifier": "test.lan",
            "log_type": "result",
            "name": "pack_incident-response_listening_ports",
            "unixTime": "1536682461",
        },
    ),
]


@panther_managed
class OsqueryMacUnwantedChromeExtensions(Rule):
    id = "Osquery.Mac.UnwantedChromeExtensions-prototype"
    display_name = "OSQuery Detected Unwanted Chrome Extensions"
    log_types = [LogType.OSQUERY_DIFFERENTIAL]
    tags = ["Osquery", "MacOS", "Malware", "Persistence:Browser Extensions"]
    reports = {"MITRE ATT&CK": ["TA0003:T1176"]}
    default_severity = Severity.MEDIUM
    default_description = "Monitor for chrome extensions that could lead to a credential compromise.\n"
    default_runbook = "Uninstall the unwanted extension"
    default_reference = "https://securelist.com/threat-in-your-browser-extensions/107181/"
    summary_attributes = ["action", "hostIdentifier", "name"]
    tests = osquery_mac_unwanted_chrome_extensions_tests

    def rule(self, event):
        return "unwanted-chrome-extensions" in event.get("name") and event.get("action") == "added"

    def title(self, event):
        return f"Unwanted Chrome extension(s) detected on [{event.get('hostIdentifier')}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get
from pypanther.helpers.panther_default import aws_cloudtrail_success

aws_cloud_trail_network_acl_permissive_entry_tests: list[RuleTest] = [
    RuleTest(
        name="Overly Permissive Entry Added",
        expected_result=True,
        log={
            "awsRegion": "us-west-2",
            "eventID": "1111",
            "eventName": "CreateNetworkAclEntry",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {
                "aclProtocol": "6",
                "cidrBlock": "0.0.0.0/0",
                "egress": False,
                "icmpTypeCode": {},
                "networkAclId": "acl-1111",
                "portRange": {"from": 700, "to": 702},
                "ruleAction": "allow",
                "ruleNumber": 12,
            },
            "responseElements": {"_return": True, "requestId": "1111"},
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Not Overly Permissive Entry Added",
        expected_result=False,
        log={
            "awsRegion": "us-west-2",
            "eventID": "1111",
            "eventName": "CreateNetworkAclEntry",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {
                "aclProtocol": "6",
                "cidrBlock": "111.111.111.111/32",
                "egress": False,
                "icmpTypeCode": {},
                "networkAclId": "acl-1111",
                "portRange": {"from": 700, "to": 702},
                "ruleAction": "allow",
                "ruleNumber": 12,
            },
            "responseElements": {"_return": True, "requestId": "1111"},
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Error Adding Overly Permissive Entry",
        expected_result=False,
        log={
            "awsRegion": "us-west-2",
            "errorCode": "ValidationError",
            "eventID": "1111",
            "eventName": "CreateNetworkAclEntry",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {
                "aclProtocol": "6",
                "cidrBlock": "0.0.0.0/0",
                "egress": False,
                "icmpTypeCode": {},
                "networkAclId": "acl-1111",
                "portRange": {"from": 700, "to": 702},
                "ruleAction": "allow",
                "ruleNumber": 12,
            },
            "responseElements": {"_return": True, "requestId": "1111"},
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
]


@panther_managed
class AWSCloudTrailNetworkACLPermissiveEntry(Rule):
    id = "AWS.CloudTrail.NetworkACLPermissiveEntry-prototype"
    display_name = "AWS Network ACL Overly Permissive Entry Created"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Persistence:Account Manipulation"]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0003:T1098"]}
    default_description = "A Network ACL entry that allows access from anywhere was added.\n"
    default_runbook = (
        "Remove the overly permissive Network ACL entry and add a new entry with more restrictive permissions.\n"
    )
    default_reference = "https://docs.aws.amazon.com/vpc/latest/userguide/vpc-network-acls.html#nacl-rules"
    summary_attributes = ["userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    tests = aws_cloud_trail_network_acl_permissive_entry_tests

    def rule(self, event):
        # Only check successful actions creating a new Network ACL entry
        if not aws_cloudtrail_success(event) or event.get("eventName") != "CreateNetworkAclEntry":
            return False
        # Check if this new NACL entry is allowing traffic from anywhere
        return (
            deep_get(event, "requestParameters", "cidrBlock") == "0.0.0.0/0"
            and deep_get(event, "requestParameters", "ruleAction") == "allow"
            and (deep_get(event, "requestParameters", "egress") is False)
        )

    def alert_context(self, event):
        return aws_rule_context(event)



import re

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp_base_helpers import gcp_alert_context
from pypanther.helpers.panther_base_helpers import deep_get

gcp_firewall_rule_deleted_tests: list[RuleTest] = [
    RuleTest(
        name="compute.firewalls-delete-should-alert",
        expected_result=True,
        log={
            "insertid": "-xxxxxxxx",
            "logname": "projects/test-project-123456/logs/cloudaudit.googleapis.com%2Factivity",
            "operation": {
                "id": "operation-1684869594486-5fc6145ac17b3-6f92b265-43256266",
                "last": True,
                "producer": "compute.googleapis.com",
            },
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@domain.com"},
                "methodName": "v1.compute.firewalls.delete",
                "request": {"@type": "type.googleapis.com/compute.firewalls.delete"},
                "requestMetadata": {
                    "callerIP": "12.12.12.12",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                },
                "resourceName": "projects/test-project-123456/global/firewalls/firewall-create",
                "serviceName": "compute.googleapis.com",
            },
            "receivetimestamp": "2023-05-23 19:20:00.728",
            "resource": {
                "labels": {"firewall_rule_id": "6563507997690081088", "project_id": "test-project-123456"},
                "type": "gce_firewall_rule",
            },
            "severity": "NOTICE",
            "timestamp": "2023-05-23 19:20:00.396",
        },
    ),
    RuleTest(
        name="appengine.firewall.delete-should-alert",
        expected_result=True,
        log={
            "insertid": "-xxxxxxxx",
            "logname": "projects/test-project-123456/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@domain.com"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "appengine.applications.update",
                        "resource": "apps/test-project-123456/firewall/ingressRules/1000",
                        "resourceAttributes": {},
                    },
                ],
                "methodName": "google.appengine.v1.Firewall.DeleteIngressRule",
                "requestMetadata": {
                    "callerIP": "12.12.12.12",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2023-05-23T19:28:48.805823Z"},
                },
                "resourceName": "apps/test-project-123456/firewall/ingressRules/1000",
                "serviceData": {"@type": "type.googleapis.com/google.appengine.v1beta4.AuditData"},
                "serviceName": "appengine.googleapis.com",
                "status": {},
            },
            "receivetimestamp": "2023-05-23 19:28:49.474",
            "resource": {
                "labels": {"module_id": "", "project_id": "test-project-123456", "version_id": "", "zone": ""},
                "type": "gae_app",
            },
            "severity": "NOTICE",
            "timestamp": "2023-05-23 19:28:48.707",
        },
    ),
    RuleTest(
        name="compute.non-delete.firewall.method-should-not-alert",
        expected_result=False,
        log={"methodName": "v1.compute.firewalls.insert"},
    ),
    RuleTest(
        name="appengine.non-delete.firewall.method-should-not-alert",
        expected_result=False,
        log={"methodName": "appengine.compute.v1.Firewall.PatchIngressRule"},
    ),
    RuleTest(
        name="randomservice.firewall-delete.method-should-alert",
        expected_result=True,
        log={
            "protoPayload": {
                "authenticationInfo": {"principalEmail": "user@domain.com"},
                "methodName": "randomservice.compute.v1.Firewall.DeleteIngressRule",
                "resourceName": "randomservice/test-project-123456/firewall/ingressRules/1000",
                "requestMetadata": {
                    "callerIP": "12.12.12.12",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2023-05-23T19:28:44.663413Z"},
                },
            },
            "resource": {
                "labels": {"firewall_rule_id": "6563507997690081088", "project_id": "test-project-123456"},
                "type": "gce_firewall_rule",
            },
        },
    ),
]


@panther_managed
class GCPFirewallRuleDeleted(Rule):
    display_name = "GCP Firewall Rule Deleted"
    id = "GCP.Firewall.Rule.Deleted-prototype"
    default_severity = Severity.LOW
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = ["GCP", "Firewall", "Networking", "Infrastructure"]
    default_description = "This rule detects deletions of GCP firewall rules.\n"
    default_runbook = "Ensure that the rule deletion was expected. Firewall rule deletions can cause service interruptions or outages.\n"
    default_reference = "https://cloud.google.com/firewall/docs/about-firewalls"
    tests = gcp_firewall_rule_deleted_tests

    def rule(self, event):
        method_pattern = "(?:\\w+\\.)*v\\d\\.(?:Firewall\\.Delete)|(compute\\.firewalls\\.delete)"
        match = re.search(method_pattern, deep_get(event, "protoPayload", "methodName", default=""))
        return match is not None

    def title(self, event):
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        resource = deep_get(event, "protoPayload", "resourceName", default="<RESOURCE_NOT_FOUND>")
        resource_id = deep_get(event, "resource", "labels", "firewall_rule_id", default="<RESOURCE_ID_NOT_FOUND>")
        if resource_id != "<RESOURCE_ID_NOT_FOUND>":
            return f"[GCP]: [{actor}] deleted firewall rule with resource ID [{resource_id}]"
        return f"[GCP]: [{actor}] deleted firewall rule for resource [{resource}]"

    def alert_context(self, event):
        return gcp_alert_context(event)



from pypanther.base import DataModel, DataModelMapping, LogType


class StandardAWSS3ServerAccess(DataModel):
    id: str = "Standard.AWS.S3ServerAccess"
    display_name: str = "AWS S3 Server Access"
    enabled: bool = True
    log_types: list[str] = [LogType.AWS_S3_SERVER_ACCESS]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="http_status", path="httpstatus"),
        DataModelMapping(name="source_ip", path="remoteip"),
        DataModelMapping(name="user_agent", path="useragent"),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

osquery_linux_mac_vulnerable_x_zliblzma_tests: list[RuleTest] = [
    RuleTest(
        name="Vulnerable liblzma",
        expected_result=True,
        log={
            "name": "pack_vuln-management_rpm_packages",
            "action": "added",
            "hostIdentifier": "test-host",
            "columns": {
                "source": "test-host",
                "name": "liblzma.so",
                "version": "5.6.1.000",
                "status": "Potentially vulnerable",
            },
        },
    ),
    RuleTest(
        name="Vulnerable xz",
        expected_result=True,
        log={
            "name": "pack_vuln-management_deb_packages",
            "action": "added",
            "hostIdentifier": "test-host",
            "columns": {
                "source": "test-host",
                "name": "xz",
                "version": "5.6.0.000",
                "status": "Potentially vulnerable",
            },
        },
    ),
    RuleTest(
        name="Not vulnerable",
        expected_result=False,
        log={
            "name": "pack_vuln-management_rpm_packages",
            "action": "added",
            "hostIdentifier": "test-host",
            "columns": {
                "source": "test-host",
                "name": "liblzma.so",
                "version": "5.4.6.000",
                "status": "Most likely not vulnerable",
            },
        },
    ),
]


@panther_managed
class OsqueryLinuxMacVulnerableXZliblzma(Rule):
    id = "Osquery.Linux.Mac.VulnerableXZliblzma-prototype"
    display_name = "A backdoored version of XZ or liblzma is vulnerable to CVE-2024-3094"
    log_types = [LogType.OSQUERY_DIFFERENTIAL]
    tags = ["Osquery", "MacOS", "Linux", "Emerging Threats", "Supply Chain Compromise"]
    reports = {"MITRE ATT&CK": ["TA0001:T1195.001"]}
    default_severity = Severity.HIGH
    default_description = "Detects vulnerable versions of XZ and liblzma on Linux and MacOS using Osquery logs. Versions 5.6.0 and 5.6.1 of xz and liblzma are most likely vulnerable to backdoor exploit. Vuln management pack must be enabled: https://github.com/osquery/osquery/blob/master/packs/vuln-management.conf\n"
    default_runbook = "Upgrade/downgrade xz and liblzma to non-vulnerable versions"
    default_reference = "https://gist.github.com/jamesspi/ee8319f55d49b4f44345c626f80c430f"
    summary_attributes = ["name", "hostIdentifier", "action"]
    tests = osquery_linux_mac_vulnerable_x_zliblzma_tests
    QUERY_NAMES = {
        "pack_vuln-management_homebrew_packages",
        "pack_vuln-management_deb_packages",
        "pack_vuln-management_rpm_packages",
    }
    VULNERABLE_PACKAGES = {"xz", "liblzma", "xz-libs", "xz-utils"}
    VULNERABLE_VERSIONS = {"5.6.0", "5.6.1"}

    def rule(self, event):
        package = event.deep_get("columns", "name", default="")
        version = event.deep_get("columns", "version", default="")
        return all(
            [
                event.get("name") in self.QUERY_NAMES,
                package in self.VULNERABLE_PACKAGES or package.startswith("liblzma"),
                any(version.startswith(v) for v in self.VULNERABLE_VERSIONS),
            ],
        )

    def title(self, event):
        host = event.get("hostIdentifier")
        name = event.deep_get("columns", "name", default="")
        version = event.deep_get("columns", "version", default="")
        return f"[CVE-2024-3094] {name} {version} Potentially vulnerable on {host}"
